@@ -1,18 +1,30 @@
 use iai::black_box;
-use shakmaty::{fen::Fen, perft, san::San, CastlingMode, Chess, Move, Position, Role, Square};
+use shakmaty::{
+    fen::Fen,
+    perft,
+    san::San,
+    zobrist::{Zobrist64, ZobristHash},
+    CastlingMode, Chess, EnPassantMode, Move, Position, Role, Square,
+};
 
 fn bench_shallow_perft() {
     let pos = Chess::default();
-    assert_eq!(perft(&pos, black_box(4)), 197_281);
+    assert_eq!(black_box(perft(black_box(&pos), 4)), 197_281);
 }
 
 fn bench_deep_perft() {
     let pos = Chess::default();
-    assert_eq!(perft(&pos, black_box(5)), 4_865_609);
+    assert_eq!(perft(black_box(&pos), 5), 4_865_609);
 }
 
-fn bench_parse_san_move_complicated() -> San {
-    San::from_ascii(black_box(b"bxc1=R+")).expect("valid san")
+fn bench_kiwipete() {
+    let fen = "r3k2r/p1ppqpb1/bn2pnp1/3PN3/1p2P3/2N2Q1p/PPPBBPPP/R3K2R w KQkq - ";
+    let pos: Chess = fen
+        .parse::<Fen>()
+        .expect("valid fen")
+        .into_position(CastlingMode::Standard)
+        .expect("legal position");
+    assert_eq!(perft(black_box(&pos), 4), 4_085_603);
 }
 
 fn bench_generate_moves() {
@@ -20,7 +32,7 @@ fn bench_generate_moves() {
     let pos: Chess = fen
         .parse::<Fen>()
         .expect("valid fen")
-        .position(CastlingMode::Chess960)
+        .into_position(CastlingMode::Chess960)
         .expect("legal position");
 
     assert_eq!(black_box(&pos).legal_moves().len(), 39);
@@ -31,7 +43,7 @@ fn bench_play_unchecked() -> Chess {
     let pos: Chess = fen
         .parse::<Fen>()
         .expect("valid fen")
-        .position(CastlingMode::Chess960)
+        .into_position(CastlingMode::Chess960)
         .expect("legal position");
 
     let m = Move::Normal {
@@ -52,7 +64,7 @@ fn bench_san_candidates() {
     let pos: Chess = fen
         .parse::<Fen>()
         .expect("valid fen")
-        .position(CastlingMode::Chess960)
+        .into_position(CastlingMode::Chess960)
         .expect("legal position");
 
     let moves = black_box(&pos).san_candidates(Role::Knight, Square::B5);
@@ -85,12 +97,17 @@ fn bench_play_sans() -> Chess {
     pos
 }
 
+fn bench_zobrist_hash() -> Zobrist64 {
+    black_box(Chess::default()).zobrist_hash(EnPassantMode::Legal)
+}
+
 iai::main!(
     bench_shallow_perft,
     bench_deep_perft,
-    bench_parse_san_move_complicated,
+    bench_kiwipete,
     bench_generate_moves,
     bench_play_unchecked,
     bench_san_candidates,
     bench_play_sans,
+    bench_zobrist_hash,
 );
