@@ -1,32 +1,13 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Sets of squares.
 
-use std::{
+use core::{
     fmt,
     fmt::Write,
     iter::{FromIterator, FusedIterator},
     ops,
 };
 
-use crate::{
-    color::Color,
-    square::{File, Rank, Square},
-};
+use crate::square::{File, Rank, Square};
 
 /// A set of [squares](super::Square) represented by a 64 bit
 /// integer mask.
@@ -52,31 +33,32 @@ pub struct Bitboard(pub u64);
 impl Bitboard {
     /// A bitboard with a single square.
     #[inline]
-    pub fn from_square(sq: Square) -> Bitboard {
-        Bitboard(SQUARES[usize::from(sq)])
+    pub const fn from_square(sq: Square) -> Bitboard {
+        Bitboard(1 << sq as usize)
     }
 
     /// Returns the bitboard containing all squares of the given rank.
     #[inline]
-    pub fn from_rank(rank: Rank) -> Bitboard {
-        Bitboard(RANKS[usize::from(rank)])
+    pub const fn from_rank(rank: Rank) -> Bitboard {
+        Bitboard(RANKS[rank as usize])
     }
 
     /// Returns the bitboard containing all squares of the given file.
     #[inline]
-    pub fn from_file(file: File) -> Bitboard {
-        Bitboard(FILES[usize::from(file)])
+    pub const fn from_file(file: File) -> Bitboard {
+        Bitboard(FILE_A << file as usize)
     }
 
-    /// Shift using `<<` for `White` and `>>` for `Black`.
+    /// Silently overflowing bitwise shift with a signed offset, `<<` for
+    /// positive values and `>>` for negative values.
     ///
     /// # Examples
     ///
     /// ```
-    /// use shakmaty::{Bitboard, Color};
+    /// use shakmaty::Bitboard;
     ///
     /// let bitboard = Bitboard(0x1e22_2212_0e0a_1222);
-    /// assert_eq!(bitboard.relative_shift(Color::Black, 8), Bitboard(0x001e_2222_120e_0a12));
+    /// assert_eq!(bitboard.shift(-8), Bitboard(0x001e_2222_120e_0a12));
     /// // . . . . . . . .
     /// // . 1 1 1 1 . . .
     /// // . 1 . . . 1 . .
@@ -85,14 +67,22 @@ impl Bitboard {
     /// // . 1 1 1 . . . .
     /// // . 1 . 1 . . . .
     /// // . 1 . . 1 . . .
+    ///
+    /// assert_eq!(bitboard.shift(64), Bitboard(0));
+    /// assert_eq!(bitboard.shift(i32::MIN), Bitboard(0));
     /// ```
     #[must_use]
     #[inline]
-    pub fn relative_shift(self, color: Color, shift: u32) -> Bitboard {
-        match color {
-            Color::White => Bitboard(self.0 << shift),
-            Color::Black => Bitboard(self.0 >> shift),
-        }
+    pub const fn shift(self, offset: i32) -> Bitboard {
+        Bitboard(if offset > 63 {
+            0
+        } else if offset >= 0 {
+            self.0 << offset
+        } else if offset >= -63 {
+            self.0 >> -offset
+        } else {
+            0
+        })
     }
 
     /// Tests if `self` is non-empty.
@@ -106,7 +96,7 @@ impl Bitboard {
     /// ```
     #[must_use]
     #[inline]
-    pub fn any(self) -> bool {
+    pub const fn any(self) -> bool {
         self.0 != 0
     }
 
@@ -120,25 +110,29 @@ impl Bitboard {
     /// assert!(Bitboard::EMPTY.is_empty());
     /// ```
     #[inline]
-    pub fn is_empty(self) -> bool {
+    pub const fn is_empty(self) -> bool {
         self.0 == 0
     }
 
+    /// Tests if `self` contains the given square.
     #[inline]
-    pub fn contains(self, sq: Square) -> bool {
-        (self & Bitboard::from_square(sq)).any()
+    pub const fn contains(self, sq: Square) -> bool {
+        self.intersect(Bitboard::from_square(sq)).any()
     }
 
+    /// Adds `squares`.
     #[inline]
     pub fn add<T: Into<Bitboard>>(&mut self, squares: T) {
         *self |= squares;
     }
 
+    /// Toggles `squares`.
     #[inline]
     pub fn toggle<T: Into<Bitboard>>(&mut self, squares: T) {
         *self ^= squares;
     }
 
+    /// Discards `squares`.
     #[inline]
     pub fn discard<T: Into<Bitboard>>(&mut self, squares: T) {
         *self &= !squares.into();
@@ -169,44 +163,80 @@ impl Bitboard {
         }
     }
 
+    /// Conditionally adds or discards `square`.
     #[inline]
-    pub fn set(&mut self, sq: Square, v: bool) {
+    pub fn set(&mut self, square: Square, v: bool) {
         if v {
-            self.add(sq);
+            self.add(square);
         } else {
-            self.discard(sq);
+            self.discard(square);
         }
     }
 
+    /// Clears all squares.
     #[inline]
     pub fn clear(&mut self) {
         self.0 = 0;
     }
 
+    /// Returns the intersection of `self` and `squares`. Equivalent to bitwise `&`.
+    #[must_use]
+    pub const fn intersect(self, squares: Bitboard) -> Bitboard {
+        Bitboard(self.0 & squares.0)
+    }
+
+    /// Returns the union of `self` and `squares`. Equivalent to bitwise `|`.
     #[doc(alias = "union")]
     #[must_use]
     #[inline]
     pub fn with<T: Into<Bitboard>>(self, squares: T) -> Bitboard {
-        self | squares
+        self.with_const(squares.into())
     }
 
+    /// Same as the `with` method, but usable in `const` contexts.
+    #[must_use]
+    pub const fn with_const(self, squares: Bitboard) -> Bitboard {
+        Bitboard(self.0 | squares.0)
+    }
+
+    /// Returns `self` without `squares` (set difference).
     #[doc(alias = "difference")]
     #[must_use]
     #[inline]
     pub fn without<T: Into<Bitboard>>(self, squares: T) -> Bitboard {
-        self & !squares.into()
+        self.without_const(squares.into())
     }
 
+    /// Same as the `without` method, but usable in `const` contexts.
+    #[must_use]
+    pub const fn without_const(self, squares: Bitboard) -> Bitboard {
+        Bitboard(self.0 & !squares.0)
+    }
+
+    /// Returns all squares that are in `self` or `squares` but not in both
+    /// (symmetric set difference). Equivalent to bitwise `^`.
     #[doc(alias = "symmetric_difference")]
     #[must_use]
     #[inline]
     pub fn toggled<T: Into<Bitboard>>(self, squares: T) -> Bitboard {
-        self ^ squares
+        self.toggled_const(squares.into())
     }
 
+    /// Same as the `toggled` method, but usable in `const` contexts.
+    #[must_use]
+    pub const fn toggled_const(self, squares: Bitboard) -> Bitboard {
+        Bitboard(self.0 ^ squares.0)
+    }
+
+    /// Tests if `self` and `other` are disjoint.
     #[inline]
     pub fn is_disjoint<T: Into<Bitboard>>(self, other: T) -> bool {
-        (self & other).is_empty()
+        self.is_disjoint_const(other.into())
+    }
+
+    /// Same as the `is_disjoint` method, but usable in `const` contexts.
+    pub const fn is_disjoint_const(self, other: Bitboard) -> bool {
+        Bitboard(self.0 & other.0).is_empty()
     }
 
     /// Tests if `self` is a subset of `other`.
@@ -220,7 +250,12 @@ impl Bitboard {
     /// ```
     #[inline]
     pub fn is_subset<T: Into<Bitboard>>(self, other: T) -> bool {
-        (self & !other.into()).is_empty()
+        self.is_subset_const(other.into())
+    }
+
+    /// Same as the `is_subset` method, but usable in `const` contexts.
+    pub const fn is_subset_const(self, other: Bitboard) -> bool {
+        self.without_const(other).is_empty()
     }
 
     /// Tests if `self` is a superset of `other`.
@@ -234,18 +269,26 @@ impl Bitboard {
     /// ```
     #[inline]
     pub fn is_superset<T: Into<Bitboard>>(self, other: T) -> bool {
-        other.into().is_subset(self)
+        self.is_superset_const(other.into())
     }
 
+    /// Same as the `is_superset` method, but usable in `const` contexts.
+    pub const fn is_superset_const(self, other: Bitboard) -> bool {
+        other.is_subset_const(self)
+    }
+
+    /// Removes and returns the first square, if any.
+    #[must_use = "use Bitboard::discard_first() if return value is not needed"]
     #[inline]
     pub fn pop_front(&mut self) -> Option<Square> {
         let square = self.first();
-        self.0 &= self.0.wrapping_sub(1);
+        self.discard_first();
         square
     }
 
+    /// Returns the first square, if any.
     #[inline]
-    pub fn first(self) -> Option<Square> {
+    pub const fn first(self) -> Option<Square> {
         if self.is_empty() {
             None
         } else {
@@ -253,6 +296,29 @@ impl Bitboard {
         }
     }
 
+    /// Discards the first square.
+    #[inline]
+    pub fn discard_first(&mut self) {
+        *self = self.without_first();
+    }
+
+    /// Returns `self` without the first square.
+    #[must_use]
+    #[inline]
+    pub const fn without_first(self) -> Bitboard {
+        let Bitboard(mask) = self;
+        Bitboard(mask & mask.wrapping_sub(1))
+    }
+
+    /// Returns the bitboard with only the first square of `self`.
+    #[must_use]
+    #[inline]
+    pub const fn isolate_first(self) -> Bitboard {
+        let Bitboard(mask) = self;
+        Bitboard(mask & mask.wrapping_neg())
+    }
+
+    /// Removes and returns the last square, if any.
     #[inline]
     pub fn pop_back(&mut self) -> Option<Square> {
         let square = self.last();
@@ -260,8 +326,9 @@ impl Bitboard {
         square
     }
 
+    /// Returns the last square.
     #[inline]
-    pub fn last(self) -> Option<Square> {
+    pub const fn last(self) -> Option<Square> {
         if self.is_empty() {
             None
         } else {
@@ -269,20 +336,59 @@ impl Bitboard {
         }
     }
 
+    /// Discards the last square.
+    #[inline]
+    pub fn discard_last(&mut self) {
+        *self = self.without_last()
+    }
+
+    /// Returns `self` without the last square.
+    #[must_use]
+    #[inline]
+    pub fn without_last(self) -> Bitboard {
+        self ^ Bitboard::from_iter(self.last())
+    }
+
+    /// Returns the bitboard with only the last square of `self`.
+    #[must_use]
+    #[inline]
+    pub fn isolate_last(self) -> Bitboard {
+        Bitboard::from_iter(self.last())
+    }
+
+    /// Returns the number of squares in `self`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use shakmaty::Bitboard;
+    ///
+    /// assert_eq!(Bitboard::CORNERS.count(), 4);
+    /// ```
     #[doc(alias = "len")]
     #[inline]
-    pub fn count(self) -> usize {
+    pub const fn count(self) -> usize {
         self.0.count_ones() as usize
     }
 
+    /// Tests if there is more than one square in `self`.
     #[inline]
-    pub fn more_than_one(self) -> bool {
-        self.0 & self.0.wrapping_sub(1) != 0
+    pub const fn more_than_one(self) -> bool {
+        self.without_first().any()
     }
 
     /// Gets the only square in the set, if there is exactly one.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use shakmaty::{Bitboard, Square, Rank};
+    ///
+    /// assert_eq!(Bitboard::from(Square::H5).single_square(), Some(Square::H5));
+    /// assert_eq!(Bitboard::from(Rank::First).single_square(), None);
+    /// ```
     #[inline]
-    pub fn single_square(self) -> Option<Square> {
+    pub const fn single_square(self) -> Option<Square> {
         if self.more_than_one() {
             None
         } else {
@@ -291,8 +397,18 @@ impl Bitboard {
     }
 
     /// An iterator over the subsets of this bitboard.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use shakmaty::Bitboard;
+    ///
+    /// for subset in Bitboard::CENTER.carry_rippler() {
+    ///     assert!(subset.is_subset(Bitboard::CENTER));
+    /// }
+    /// ```
     #[inline]
-    pub fn carry_rippler(self) -> CarryRippler {
+    pub const fn carry_rippler(self) -> CarryRippler {
         CarryRippler {
             bb: self.0,
             subset: 0,
@@ -320,7 +436,7 @@ impl Bitboard {
     /// ```
     #[must_use]
     #[inline]
-    pub fn flip_vertical(self) -> Bitboard {
+    pub const fn flip_vertical(self) -> Bitboard {
         Bitboard(self.0.swap_bytes())
     }
 
@@ -343,7 +459,7 @@ impl Bitboard {
     /// // . . 1 . . . 1 .
     /// ```
     #[must_use]
-    pub fn flip_horizontal(self) -> Bitboard {
+    pub const fn flip_horizontal(self) -> Bitboard {
         // https://www.chessprogramming.org/Flipping_Mirroring_and_Rotating#Horizontal
         let k1 = 0x5555_5555_5555_5555;
         let k2 = 0x3333_3333_3333_3333;
@@ -374,7 +490,7 @@ impl Bitboard {
     /// // . . . . . . . .
     /// ```
     #[must_use]
-    pub fn flip_diagonal(self) -> Bitboard {
+    pub const fn flip_diagonal(self) -> Bitboard {
         // https://www.chessprogramming.org/Flipping_Mirroring_and_Rotating#Diagonal
         let k1 = 0x5500_5500_5500_5500;
         let k2 = 0x3333_0000_3333_0000;
@@ -408,7 +524,7 @@ impl Bitboard {
     /// // . . . . . . . .
     /// ```
     #[must_use]
-    pub fn flip_anti_diagonal(self) -> Bitboard {
+    pub const fn flip_anti_diagonal(self) -> Bitboard {
         // https://www.chessprogramming.org/Flipping_Mirroring_and_Rotating#Anti-Diagonal
         let k1 = 0xaa00_aa00_aa00_aa00;
         let k2 = 0xcccc_0000_cccc_0000;
@@ -442,7 +558,7 @@ impl Bitboard {
     /// // . . . . . . . .
     /// ```
     #[must_use]
-    pub fn rotate_90(self) -> Bitboard {
+    pub const fn rotate_90(self) -> Bitboard {
         self.flip_diagonal().flip_vertical()
     }
 
@@ -466,7 +582,7 @@ impl Bitboard {
     /// ```
     #[must_use]
     #[inline]
-    pub fn rotate_180(self) -> Bitboard {
+    pub const fn rotate_180(self) -> Bitboard {
         Bitboard(self.0.reverse_bits())
     }
 
@@ -489,7 +605,7 @@ impl Bitboard {
     /// // . . . . . . . .
     /// ```
     #[must_use]
-    pub fn rotate_270(self) -> Bitboard {
+    pub const fn rotate_270(self) -> Bitboard {
         self.flip_vertical().flip_diagonal()
     }
 
@@ -681,19 +797,8 @@ impl Bitboard {
     pub const EAST: Bitboard = Bitboard(0xf0f0_f0f0_f0f0_f0f0);
 }
 
-/// Square masks.
-static SQUARES: [u64; 64] = {
-    let mut masks = [0; 64];
-    let mut i = 0;
-    while i < 64 {
-        masks[i] = 1 << i;
-        i += 1;
-    }
-    masks
-};
-
 /// Rank masks.
-static RANKS: [u64; 8] = {
+const RANKS: [u64; 8] = {
     let mut masks = [0; 8];
     let mut i = 0;
     while i < 8 {
@@ -703,16 +808,7 @@ static RANKS: [u64; 8] = {
     masks
 };
 
-/// File masks.
-static FILES: [u64; 8] = {
-    let mut masks = [0; 8];
-    let mut i = 0;
-    while i < 8 {
-        masks[i] = 0x0101_0101_0101_0101 << i;
-        i += 1;
-    }
-    masks
-};
+const FILE_A: u64 = 0x0101_0101_0101_0101;
 
 #[derive(Copy, Clone)]
 pub(crate) enum Direction {
@@ -724,7 +820,7 @@ pub(crate) enum Direction {
 
 impl Direction {
     #[inline(always)]
-    pub fn offset(self) -> i32 {
+    pub const fn offset(self) -> i32 {
         match self {
             Direction::NorthWest => 7,
             Direction::SouthWest => -9,
@@ -734,12 +830,12 @@ impl Direction {
     }
 
     #[inline(always)]
-    pub fn translate(self, bitboard: Bitboard) -> Bitboard {
+    pub const fn translate(self, bitboard: Bitboard) -> Bitboard {
         Bitboard(match self {
-            Direction::NorthWest => (bitboard.0 & !FILES[0]) << 7,
-            Direction::SouthWest => (bitboard.0 & !FILES[0]) >> 9,
-            Direction::NorthEast => (bitboard.0 & !FILES[7]) << 9,
-            Direction::SouthEast => (bitboard.0 & !FILES[7]) >> 7,
+            Direction::NorthWest => (bitboard.0 & !FILE_A) << 7,
+            Direction::SouthWest => (bitboard.0 & !FILE_A) >> 9,
+            Direction::NorthEast => (bitboard.0 << 9) & !FILE_A,
+            Direction::SouthEast => (bitboard.0 >> 7) & !FILE_A,
         })
     }
 }
@@ -899,7 +995,7 @@ impl ops::Not for Bitboard {
 }
 
 impl FromIterator<Square> for Bitboard {
-    fn from_iter<T>(iter: T) -> Self
+    fn from_iter<T>(iter: T) -> Bitboard
     where
         T: IntoIterator<Item = Square>,
     {
@@ -973,6 +1069,8 @@ impl DoubleEndedIterator for IntoIter {
 }
 
 /// Iterator over the subsets of a [`Bitboard`].
+///
+/// See [`Bitboard::carry_rippler()`].
 #[derive(Debug, Clone)]
 pub struct CarryRippler {
     bb: u64,
@@ -1014,15 +1112,18 @@ impl FusedIterator for CarryRippler {}
 
 #[cfg(test)]
 mod tests {
+    #[cfg(feature = "alloc")]
+    use alloc::format;
+
     use super::*;
 
     #[test]
     fn test_more_than_one() {
-        assert_eq!(Bitboard(0).more_than_one(), false);
-        assert_eq!(Bitboard(1).more_than_one(), false);
-        assert_eq!(Bitboard(2).more_than_one(), false);
-        assert_eq!(Bitboard(3).more_than_one(), true);
-        assert_eq!(Bitboard::FULL.more_than_one(), true);
+        assert!(!Bitboard(0).more_than_one());
+        assert!(!Bitboard(1).more_than_one());
+        assert!(!Bitboard(2).more_than_one());
+        assert!(Bitboard(3).more_than_one());
+        assert!(Bitboard::FULL.more_than_one());
     }
 
     #[test]
@@ -1042,6 +1143,24 @@ mod tests {
         assert_eq!(Bitboard(0).last(), None);
     }
 
+    #[test]
+    fn test_isolate_first() {
+        assert_eq!(
+            Bitboard::from(Rank::Second).isolate_first(),
+            Bitboard::from_square(Square::A2)
+        );
+        assert_eq!(Bitboard(0).isolate_first(), Bitboard(0));
+    }
+
+    #[test]
+    fn test_isolate_last() {
+        assert_eq!(
+            Bitboard::from(File::C).isolate_last(),
+            Bitboard::from_square(Square::C8)
+        );
+        assert_eq!(Bitboard(0).isolate_last(), Bitboard(0));
+    }
+
     #[test]
     fn test_is_empty() {
         assert!(Bitboard(0).is_empty());
@@ -1062,21 +1181,25 @@ mod tests {
         );
     }
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_upper_hex() {
         assert_eq!(format!("{:#0X}", Bitboard(42)), format!("{:#0X}", 42));
     }
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_lower_hex() {
         assert_eq!(format!("{:#0x}", Bitboard(42)), format!("{:#0x}", 42));
     }
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_octal() {
         assert_eq!(format!("{:#0o}", Bitboard(42)), format!("{:#0o}", 42));
     }
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_binary() {
         assert_eq!(format!("{:#0b}", Bitboard(42)), format!("{:#0b}", 42));
