@@ -1,22 +1,6 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Piece positions on a board.
 
-use std::{
+use core::{
     fmt,
     fmt::Write,
     iter::{FromIterator, FusedIterator},
@@ -51,7 +35,7 @@ pub struct Board {
 }
 
 impl Board {
-    pub fn new() -> Board {
+    pub const fn new() -> Board {
         Board {
             by_role: ByRole {
                 pawn: Bitboard(0x00ff_0000_0000_ff00),
@@ -69,16 +53,58 @@ impl Board {
         }
     }
 
-    pub fn empty() -> Board {
+    pub const fn empty() -> Board {
         Board {
-            by_role: ByRole::default(),
-            by_color: ByColor::default(),
+            by_role: ByRole {
+                pawn: Bitboard::EMPTY,
+                knight: Bitboard::EMPTY,
+                bishop: Bitboard::EMPTY,
+                rook: Bitboard::EMPTY,
+                queen: Bitboard::EMPTY,
+                king: Bitboard::EMPTY,
+            },
+            by_color: ByColor {
+                white: Bitboard::EMPTY,
+                black: Bitboard::EMPTY,
+            },
             occupied: Bitboard::EMPTY,
         }
     }
 
+    /// Creates a board from bitboard constituents.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the bitboards are inconsistent.
+    #[track_caller]
+    pub fn from_bitboards(by_role: ByRole<Bitboard>, by_color: ByColor<Bitboard>) -> Board {
+        let mut occupied = Bitboard::EMPTY;
+        by_role.for_each(|role| {
+            assert!(occupied.is_disjoint(role), "by_role not disjoint");
+            occupied |= role;
+        });
+        assert!(
+            by_color.black.is_disjoint(by_color.white),
+            "by_color not disjoint"
+        );
+        assert_eq!(
+            occupied,
+            by_color.black | by_color.white,
+            "by_role does not match by_color"
+        );
+        Board {
+            by_role,
+            by_color,
+            occupied,
+        }
+    }
+
+    pub const fn into_bitboards(self) -> (ByRole<Bitboard>, ByColor<Bitboard>) {
+        (self.by_role, self.by_color)
+    }
+
     #[cfg(feature = "variant")]
-    pub fn racing_kings() -> Board {
+    pub const fn racing_kings() -> Board {
         Board {
             by_role: ByRole {
                 pawn: Bitboard(0x0000),
@@ -97,7 +123,7 @@ impl Board {
     }
 
     #[cfg(feature = "variant")]
-    pub fn horde() -> Board {
+    pub const fn horde() -> Board {
         Board {
             by_role: ByRole {
                 pawn: Bitboard(0x00ff_0066_ffff_ffff),
@@ -116,68 +142,89 @@ impl Board {
     }
 
     #[inline]
-    pub fn occupied(&self) -> Bitboard {
+    pub const fn occupied(&self) -> Bitboard {
         self.occupied
     }
 
     #[inline]
-    pub fn pawns(&self) -> Bitboard {
+    pub const fn pawns(&self) -> Bitboard {
         self.by_role.pawn
     }
+
     #[inline]
-    pub fn knights(&self) -> Bitboard {
+    pub const fn knights(&self) -> Bitboard {
         self.by_role.knight
     }
+
     #[inline]
-    pub fn bishops(&self) -> Bitboard {
+    pub const fn bishops(&self) -> Bitboard {
         self.by_role.bishop
     }
+
     #[inline]
-    pub fn rooks(&self) -> Bitboard {
+    pub const fn rooks(&self) -> Bitboard {
         self.by_role.rook
     }
+
     #[inline]
-    pub fn queens(&self) -> Bitboard {
+    pub const fn queens(&self) -> Bitboard {
         self.by_role.queen
     }
+
     #[inline]
-    pub fn kings(&self) -> Bitboard {
+    pub const fn kings(&self) -> Bitboard {
         self.by_role.king
     }
 
     #[inline]
-    pub fn white(&self) -> Bitboard {
+    pub const fn white(&self) -> Bitboard {
         self.by_color.white
     }
+
     #[inline]
-    pub fn black(&self) -> Bitboard {
+    pub const fn black(&self) -> Bitboard {
         self.by_color.black
     }
 
     /// Bishops, rooks and queens.
     #[inline]
-    pub fn sliders(&self) -> Bitboard {
-        self.by_role.bishop ^ self.by_role.rook ^ self.by_role.queen
+    pub const fn sliders(&self) -> Bitboard {
+        let ByRole {
+            bishop,
+            rook,
+            queen,
+            ..
+        } = self.by_role;
+        bishop.toggled_const(rook).toggled_const(queen)
     }
+
     /// Pawns, knights and kings.
     #[inline]
-    pub fn steppers(&self) -> Bitboard {
-        self.by_role.pawn ^ self.by_role.knight ^ self.by_role.king
+    pub const fn steppers(&self) -> Bitboard {
+        let ByRole {
+            pawn, knight, king, ..
+        } = self.by_role;
+        pawn.toggled_const(knight).toggled_const(king)
     }
 
     #[inline]
-    pub fn rooks_and_queens(&self) -> Bitboard {
-        self.by_role.rook ^ self.by_role.queen
+    pub const fn rooks_and_queens(&self) -> Bitboard {
+        let ByRole { rook, queen, .. } = self.by_role;
+        rook.toggled_const(queen)
     }
     #[inline]
-    pub fn bishops_and_queens(&self) -> Bitboard {
-        self.by_role.bishop ^ self.by_role.queen
+    pub const fn bishops_and_queens(&self) -> Bitboard {
+        let ByRole { bishop, queen, .. } = self.by_role;
+        bishop.toggled_const(queen)
     }
 
-    /// The (unique) king of the given side.
+    /// The (unique!) king of the given side, if any.
     #[inline]
-    pub fn king_of(&self, color: Color) -> Option<Square> {
-        (self.by_role.king & self.by_color(color)).single_square()
+    pub const fn king_of(&self, color: Color) -> Option<Square> {
+        self.by_role
+            .king
+            .intersect(self.by_color(color))
+            .single_square()
     }
 
     #[inline]
@@ -187,10 +234,10 @@ impl Board {
 
     #[inline]
     pub fn role_at(&self, sq: Square) -> Option<Role> {
-        if !self.occupied.contains(sq) {
-            None // catch early
-        } else {
+        if self.occupied.contains(sq) {
             self.by_role.find(|r| r.contains(sq))
+        } else {
+            None // catch early
         }
     }
 
@@ -230,18 +277,19 @@ impl Board {
     }
 
     #[inline]
-    pub fn by_color(&self, color: Color) -> Bitboard {
+    pub const fn by_color(&self, color: Color) -> Bitboard {
         *self.by_color.get(color)
     }
 
     #[inline]
-    pub fn by_role(&self, role: Role) -> Bitboard {
+    pub const fn by_role(&self, role: Role) -> Bitboard {
         *self.by_role.get(role)
     }
 
     #[inline]
-    pub fn by_piece(&self, piece: Piece) -> Bitboard {
-        self.by_color(piece.color) & self.by_role(piece.role)
+    pub const fn by_piece(&self, piece: Piece) -> Bitboard {
+        self.by_color(piece.color)
+            .intersect(self.by_role(piece.role))
     }
 
     pub fn attacks_from(&self, sq: Square) -> Bitboard {
@@ -257,7 +305,7 @@ impl Board {
                 | (attacks::bishop_attacks(sq, occupied) & self.bishops_and_queens())
                 | (attacks::knight_attacks(sq) & self.by_role.knight)
                 | (attacks::king_attacks(sq) & self.by_role.king)
-                | (attacks::pawn_attacks(!attacker, sq) & self.by_role.pawn))
+                | (attacks::pawn_attacks(attacker.other(), sq) & self.by_role.pawn))
     }
 
     pub fn material_side(&self, color: Color) -> ByRole<u8> {
@@ -333,7 +381,7 @@ impl Board {
 }
 
 impl Default for Board {
-    fn default() -> Self {
+    fn default() -> Board {
         Board::new()
     }
 }
@@ -361,7 +409,7 @@ impl Extend<(Square, Piece)> for Board {
 }
 
 impl FromIterator<(Square, Piece)> for Board {
-    fn from_iter<T>(iter: T) -> Self
+    fn from_iter<T>(iter: T) -> Board
     where
         T: IntoIterator<Item = (Square, Piece)>,
     {
@@ -484,4 +532,10 @@ mod tests {
         );
         compare_trans(&Board::rotate_270, "8/8/7N/1np3B1/b2K1R2/r3Q3/qkPNBRQK/8");
     }
+
+    #[test]
+    fn test_from_bitboards() {
+        let (by_role, by_color) = Board::default().into_bitboards();
+        assert_eq!(Board::default(), Board::from_bitboards(by_role, by_color));
+    }
 }
