@@ -1,22 +1,6 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! White or black.
 
-use std::{array, error::Error, fmt, mem, ops, str::FromStr};
+use core::{array, convert::identity, fmt, mem, ops, str::FromStr};
 
 use crate::{
     role::{ByRole, Role},
@@ -33,7 +17,7 @@ pub enum Color {
 }
 
 impl Color {
-    pub fn from_char(ch: char) -> Option<Color> {
+    pub const fn from_char(ch: char) -> Option<Color> {
         Some(match ch {
             'w' => Color::White,
             'b' => Color::Black,
@@ -53,12 +37,15 @@ impl Color {
         })
     }
 
-    fn name(self) -> &'static str {
-        self.fold_wb("white", "black")
+    const fn name(self) -> &'static str {
+        match self {
+            Color::Black => "black",
+            Color::White => "white",
+        }
     }
 
     #[inline]
-    pub fn from_white(white: bool) -> Color {
+    pub const fn from_white(white: bool) -> Color {
         if white {
             Color::White
         } else {
@@ -67,7 +54,7 @@ impl Color {
     }
 
     #[inline]
-    pub fn from_black(black: bool) -> Color {
+    pub const fn from_black(black: bool) -> Color {
         if black {
             Color::Black
         } else {
@@ -84,17 +71,29 @@ impl Color {
     }
 
     #[inline]
-    pub fn is_white(self) -> bool {
-        self == Color::White
+    pub const fn is_white(self) -> bool {
+        matches!(self, Color::White)
     }
     #[inline]
-    pub fn is_black(self) -> bool {
-        self == Color::Black
+    pub const fn is_black(self) -> bool {
+        matches!(self, Color::Black)
+    }
+
+    /// Same as `!self`, but usable in `const` contexts.
+    #[must_use]
+    pub const fn other(self) -> Color {
+        match self {
+            Color::White => Color::Black,
+            Color::Black => Color::White,
+        }
     }
 
     #[inline]
-    pub fn backrank(self) -> Rank {
-        self.fold_wb(Rank::First, Rank::Eighth)
+    pub const fn backrank(self) -> Rank {
+        match self {
+            Color::White => Rank::First,
+            Color::Black => Rank::Eighth,
+        }
     }
 
     #[inline]
@@ -106,27 +105,32 @@ impl Color {
     }
 
     #[inline]
-    pub fn pawn(self) -> Piece {
+    pub const fn pawn(self) -> Piece {
         Role::Pawn.of(self)
     }
+
     #[inline]
-    pub fn knight(self) -> Piece {
+    pub const fn knight(self) -> Piece {
         Role::Knight.of(self)
     }
+
     #[inline]
-    pub fn bishop(self) -> Piece {
+    pub const fn bishop(self) -> Piece {
         Role::Bishop.of(self)
     }
+
     #[inline]
-    pub fn rook(self) -> Piece {
+    pub const fn rook(self) -> Piece {
         Role::Rook.of(self)
     }
+
     #[inline]
-    pub fn queen(self) -> Piece {
+    pub const fn queen(self) -> Piece {
         Role::Queen.of(self)
     }
+
     #[inline]
-    pub fn king(self) -> Piece {
+    pub const fn king(self) -> Piece {
         Role::King.of(self)
     }
 
@@ -152,6 +156,12 @@ impl ops::BitXor<bool> for Color {
     }
 }
 
+impl ops::BitXorAssign<bool> for Color {
+    fn bitxor_assign(&mut self, flip: bool) {
+        *self = *self ^ flip;
+    }
+}
+
 impl fmt::Display for Color {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str(self.name())
@@ -168,7 +178,8 @@ impl fmt::Display for ParseColorError {
     }
 }
 
-impl Error for ParseColorError {}
+#[cfg(feature = "std")]
+impl std::error::Error for ParseColorError {}
 
 impl FromStr for Color {
     type Err = ParseColorError;
@@ -179,7 +190,7 @@ impl FromStr for Color {
 }
 
 /// Container with values for each [`Color`].
-#[derive(Clone, Default, Eq, PartialEq, Debug, Hash)]
+#[derive(Copy, Clone, Default, Eq, PartialEq, Debug, Hash)]
 #[repr(C)]
 pub struct ByColor<T> {
     pub black: T,
@@ -199,7 +210,7 @@ impl<T> ByColor<T> {
     }
 
     #[inline]
-    pub fn get(&self, color: Color) -> &T {
+    pub const fn get(&self, color: Color) -> &T {
         // Safety: Trivial offset into #[repr(C)] struct.
         unsafe {
             &*(self as *const ByColor<T>)
@@ -261,7 +272,7 @@ impl<T> ByColor<T> {
     }
 
     #[inline]
-    pub fn as_ref(&self) -> ByColor<&T> {
+    pub const fn as_ref(&self) -> ByColor<&T> {
         ByColor {
             black: &self.black,
             white: &self.white,
@@ -276,13 +287,6 @@ impl<T> ByColor<T> {
         }
     }
 
-    pub fn zip_color(self) -> ByColor<(Color, T)> {
-        ByColor {
-            black: (Color::Black, self.black),
-            white: (Color::White, self.white),
-        }
-    }
-
     pub fn zip<U>(self, other: ByColor<U>) -> ByColor<(T, U)> {
         ByColor {
             black: (self.black, other.black),
@@ -290,6 +294,10 @@ impl<T> ByColor<T> {
         }
     }
 
+    pub fn zip_color(self) -> ByColor<(Color, T)> {
+        ByColor::new_with(identity).zip(self)
+    }
+
     pub fn iter(&self) -> array::IntoIter<&T, 2> {
         self.as_ref().into_iter()
     }
@@ -300,7 +308,7 @@ impl<T> ByColor<T> {
 }
 
 impl<T> ByColor<ByRole<T>> {
-    pub fn piece(&self, piece: Piece) -> &T {
+    pub const fn piece(&self, piece: Piece) -> &T {
         self.get(piece.color).get(piece.role)
     }
 
@@ -312,7 +320,7 @@ impl<T> ByColor<ByRole<T>> {
 #[cfg(feature = "variant")]
 impl ByColor<ByRole<u8>> {
     pub(crate) fn count(&self) -> usize {
-        self.iter().map(|side| side.count()).sum()
+        self.iter().map(ByRole::count).sum()
     }
 }
 
