@@ -1,19 +1,3 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Parse and write Forsyth-Edwards-Notation.
 //!
 //! # Parsing
@@ -21,11 +5,14 @@
 //! The parser is relaxed:
 //!
 //! * Supports X-FEN and Shredder-FEN for castling right notation.
-//!   Allows repeated castling rights and castling rights in any order.
+//!   - Ignores repeated castling rights.
+//!   - Allows castling rights in any order.
+//!   - Allows castling rights without matching rooks (treating `Q` and `K` as
+//!     `A` and `H` respectively).
 //! * Supports `[q]` and `/q` styles for Crazyhouse pockets.
 //! * Supports `3+3` and `+0+0` for remaining checks in Three-Check.
-//! * Accepts partial FENs and fills missing fields with the default values
-//!   of `8/8/8/8/8/8/8/8 w - - 0 1`.
+//! * Accepts missing FEN fields (except the board) and fills them with
+//!   default values of `8/8/8/8/8/8/8/8 w - - 0 1`.
 //! * Accepts multiple spaces and underscores (`_`) as separators between
 //!   FEN fields.
 //! * Accepts `0` as fulllmove number and uses `1` instead.
@@ -36,10 +23,14 @@
 //! use shakmaty::{fen::Fen, CastlingMode, Chess, Position};
 //!
 //! let fen: Fen = "r1bqkbnr/ppp2Qpp/2np4/4p3/2B1P3/8/PPPP1PPP/RNB1K1NR b KQkq - 0 4".parse()?;
-//!
 //! let pos: Chess = fen.into_position(CastlingMode::Standard)?;
 //! assert!(pos.is_checkmate());
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//!
+//! # use shakmaty::{fen::ParseFenError, PositionError};
+//! # #[derive(Debug)] struct CommonError;
+//! # impl From<ParseFenError> for CommonError { fn from(_: ParseFenError) -> Self { Self } }
+//! # impl<P> From<PositionError<P>> for CommonError { fn from(_: PositionError<P>) -> Self { Self } }
+//! # Ok::<_, CommonError>(())
 //! ```
 //!
 //! # Writing
@@ -58,11 +49,10 @@
 //!            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -");
 //! ```
 
-use std::{
+use core::{
     char,
     cmp::max,
     convert::TryFrom,
-    error::Error,
     fmt,
     fmt::{Display, Write as _},
     num::NonZeroU32,
@@ -70,9 +60,8 @@ use std::{
 };
 
 use crate::{
-    Bitboard, Board, ByColor, ByRole, CastlingMode, Color, File, FromSetup, Piece, PositionError,
-    Rank, RemainingChecks, Role, Setup, Square, EnPassantMode,
-    Position,
+    Bitboard, Board, ByColor, ByRole, CastlingMode, Color, EnPassantMode, File, FromSetup, Piece,
+    Position, PositionError, Rank, RemainingChecks, Role, Setup, Square,
 };
 
 fn fmt_castling(
@@ -83,11 +72,13 @@ fn fmt_castling(
     let mut empty = true;
 
     for color in Color::ALL {
-        let king = board.king_of(color);
+        let king = board
+            .king_of(color)
+            .filter(|k| k.rank() == color.backrank());
 
         let candidates = board.by_piece(color.rook()) & color.backrank();
 
-        for rook in (candidates & castling_rights).into_iter().rev() {
+        for rook in (castling_rights & color.backrank()).into_iter().rev() {
             f.write_char(
                 if Some(rook) == candidates.first() && king.map_or(false, |k| rook < k) {
                     color.fold_wb('Q', 'q')
@@ -123,7 +114,7 @@ fn fmt_pockets(f: &mut fmt::Formatter<'_>, pockets: &ByColor<ByRole<u8>>) -> fmt
 }
 
 fn fmt_epd(f: &mut fmt::Formatter<'_>, setup: &Setup) -> fmt::Result {
-    f.write_str(&setup.board.board_fen(setup.promoted))?;
+    setup.board.board_fen(setup.promoted).fmt(f)?;
     if let Some(ref pockets) = setup.pockets {
         fmt_pockets(f, pockets)?;
     }
@@ -173,7 +164,8 @@ impl Display for ParseFenError {
     }
 }
 
-impl Error for ParseFenError {}
+#[cfg(feature = "std")]
+impl std::error::Error for ParseFenError {}
 
 fn parse_board_fen(board_fen: &[u8]) -> Result<(Board, Bitboard), ParseFenError> {
     let mut promoted = Bitboard(0);
@@ -263,45 +255,13 @@ impl Board {
     /// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR`.
     ///
     /// Promoted pieces are marked like `Q~`.
-    pub fn board_fen(&self, promoted: Bitboard) -> String {
-        let mut fen = String::with_capacity(15);
-
-        for rank in Rank::ALL.into_iter().rev() {
-            let mut empty = 0;
-
-            for file in File::ALL {
-                let square = Square::from_coords(file, rank);
-
-                empty = self.piece_at(square).map_or_else(
-                    || empty + 1,
-                    |piece| {
-                        if empty > 0 {
-                            fen.push(
-                                char::from_digit(empty, 10)
-                                    .expect("at most 8 empty squares on a rank"),
-                            );
-                        }
-                        fen.push(piece.char());
-                        if promoted.contains(square) {
-                            fen.push('~');
-                        }
-                        0
-                    },
-                );
-
-                if file == File::H && empty > 0 {
-                    fen.push(
-                        char::from_digit(empty, 10).expect("at most 8 empty squares on a rank"),
-                    );
-                }
-
-                if file == File::H && rank > Rank::First {
-                    fen.push('/');
-                }
-            }
+    ///
+    /// Returns a [`BoardFen`] which implements [`Display`].
+    pub const fn board_fen(&self, promoted: Bitboard) -> BoardFen<'_> {
+        BoardFen {
+            board: self,
+            promoted,
         }
-
-        fen
     }
 }
 
@@ -315,7 +275,54 @@ impl FromStr for Board {
 
 impl Display for Board {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(&self.board_fen(Bitboard(0)))
+        self.board_fen(Bitboard(0)).fmt(f)
+    }
+}
+
+/// Displays a board with notation like
+/// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR`.
+///
+/// See [`Board::board_fen`].
+#[derive(Debug)]
+pub struct BoardFen<'b> {
+    board: &'b Board,
+    promoted: Bitboard,
+}
+
+impl<'b> Display for BoardFen<'b> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        for rank in Rank::ALL.into_iter().rev() {
+            let mut empty = 0;
+
+            for file in File::ALL {
+                let square = Square::from_coords(file, rank);
+
+                empty = if let Some(piece) = self.board.piece_at(square) {
+                    if empty > 0 {
+                        let ch = char::from_digit(empty, 10).ok_or(fmt::Error)?;
+                        f.write_char(ch)?;
+                    }
+                    f.write_char(piece.char())?;
+                    if self.promoted.contains(square) {
+                        f.write_char('~')?;
+                    }
+                    0
+                } else {
+                    empty + 1
+                };
+
+                if file == File::H && empty > 0 {
+                    let ch = char::from_digit(empty, 10).ok_or(fmt::Error)?;
+                    f.write_char(ch)?;
+                }
+
+                if file == File::H && rank > Rank::First {
+                    f.write_char('/')?;
+                }
+            }
+        }
+
+        Ok(())
     }
 }
 
@@ -325,14 +332,10 @@ pub struct Fen(pub Setup);
 
 impl Fen {
     /// The FEN of the empty position `8/8/8/8/8/8/8/8 w - - 0 1`.
-    pub fn empty() -> Fen {
+    pub const fn empty() -> Fen {
         Fen(Setup::empty())
     }
 
-    pub fn from_position<P: Position>(pos: P, mode: EnPassantMode) -> Fen {
-        Fen(pos.into_setup(mode))
-    }
-
     /// Parses a FEN or EPD.
     ///
     /// FENs consist of parts separated by spaces. This parser also accepts
@@ -349,7 +352,7 @@ impl Fen {
     ///
     /// let fen = Fen::from_ascii(b"rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1")?;
     /// assert_eq!(fen, Fen::default());
-    /// # Ok::<_, Box<dyn std::error::Error>>(())
+    /// # Ok::<_, shakmaty::fen::ParseFenError>(())
     /// ```
     pub fn from_ascii(fen: &[u8]) -> Result<Fen, ParseFenError> {
         let mut result = Setup::empty();
@@ -370,7 +373,7 @@ impl Fen {
         } else if let Some(split_point) = board_part
             .iter()
             .enumerate()
-            .filter_map(|(idx, ch)| (*ch == b'/').then(|| idx))
+            .filter_map(|(idx, ch)| (*ch == b'/').then_some(idx))
             .nth(7)
         {
             // format: .../pocket
@@ -482,7 +485,19 @@ impl Fen {
         }
     }
 
-    pub fn into_setup(self) -> Setup {
+    pub const fn from_setup(setup: Setup) -> Fen {
+        Fen(setup)
+    }
+
+    pub fn from_position<P: Position>(pos: P, mode: EnPassantMode) -> Fen {
+        Fen(pos.into_setup(mode))
+    }
+
+    pub const fn as_setup(&self) -> &Setup {
+        &self.0
+    }
+
+    pub const fn into_setup(self) -> Setup {
         self.0
     }
 
@@ -499,13 +514,13 @@ impl Fen {
 
 impl From<Setup> for Fen {
     fn from(setup: Setup) -> Fen {
-        Fen(setup)
+        Fen::from_setup(setup)
     }
 }
 
 impl From<Fen> for Setup {
     fn from(fen: Fen) -> Setup {
-        fen.0
+        fen.into_setup()
     }
 }
 
@@ -529,19 +544,32 @@ impl Display for Fen {
 pub struct Epd(Setup);
 
 impl Epd {
-    pub fn empty() -> Epd {
+    pub const fn empty() -> Epd {
         Epd(Setup::empty())
     }
 
     pub fn from_ascii(epd: &[u8]) -> Result<Epd, ParseFenError> {
-        Ok(Epd::from(Fen::from_ascii(epd)?.into_setup()))
+        Ok(Epd::from_setup(Fen::from_ascii(epd)?.into_setup()))
+    }
+
+    pub const fn from_setup(mut setup: Setup) -> Epd {
+        setup.halfmoves = 0;
+        setup.fullmoves = match NonZeroU32::new(1) {
+            Some(num) => num,
+            _ => unreachable!(),
+        };
+        Epd(setup)
     }
 
     pub fn from_position<P: Position>(pos: P, mode: EnPassantMode) -> Epd {
-        Epd::from(pos.into_setup(mode))
+        Epd::from_setup(pos.into_setup(mode))
     }
 
-    pub fn into_setup(self) -> Setup {
+    pub const fn as_setup(&self) -> &Setup {
+        &self.0
+    }
+
+    pub const fn into_setup(self) -> Setup {
         self.0
     }
 
@@ -551,16 +579,14 @@ impl Epd {
 }
 
 impl From<Setup> for Epd {
-    fn from(mut setup: Setup) -> Epd {
-        setup.halfmoves = 0;
-        setup.fullmoves = NonZeroU32::new(1).unwrap();
-        Epd(setup)
+    fn from(setup: Setup) -> Epd {
+        Epd::from_setup(setup)
     }
 }
 
 impl From<Epd> for Setup {
-    fn from(Epd(setup): Epd) -> Setup {
-        setup
+    fn from(epd: Epd) -> Setup {
+        epd.into_setup()
     }
 }
 
@@ -581,10 +607,12 @@ impl Display for Epd {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{Chess, EnPassantMode, Position};
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_legal_ep_square() {
+        use alloc::string::ToString as _;
+
         let original_epd = "4k3/8/8/8/3Pp3/8/8/3KR3 b - d3";
         let fen: Fen = original_epd.parse().expect("valid fen");
         assert_eq!(
@@ -593,7 +621,7 @@ mod tests {
         );
 
         // The en passant square is not actually legal.
-        let pos: Chess = fen
+        let pos: crate::Chess = fen
             .into_position(CastlingMode::Standard)
             .expect("legal position");
         assert_eq!(pos.maybe_ep_square(), Some(Square::D3));
@@ -702,4 +730,24 @@ mod tests {
         assert_eq!(setup.halfmoves, 1);
         assert_eq!(setup.fullmoves.get(), 2);
     }
+
+    #[cfg(feature = "alloc")]
+    #[test]
+    fn test_castling_right_without_rook() {
+        use alloc::string::ToString as _;
+
+        let setup = "rRpppppp/8/8/8/8/8/PPPPPPBN/PPRQKBNR w KA"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
+        assert_eq!(
+            setup.castling_rights,
+            Bitboard::from_iter([Square::A1, Square::H1])
+        );
+
+        assert_eq!(
+            Fen(setup).to_string(),
+            "rRpppppp/8/8/8/8/8/PPPPPPBN/PPRQKBNR w KA - 0 1"
+        );
+    }
 }
