@@ -1,19 +1,3 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Read and write Standard Algebraic Notation.
 //!
 //! # Examples
@@ -25,13 +9,13 @@
 //!
 //! let san: San = "Nf3".parse()?;
 //! assert_eq!(san.to_string(), "Nf3");
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//! # Ok::<_, shakmaty::san::ParseSanError>(())
 //! ```
 //!
 //! Converting to a move:
 //!
 //! ```
-//! # use shakmaty::{Chess, Position, san::San};
+//! # use shakmaty::{Chess, Position, san::{ParseSanError, San, SanError}};
 //! use shakmaty::{Square, Role, Move};
 //! #
 //! # let san: San = "Nf3".parse()?;
@@ -45,22 +29,30 @@
 //!     to: Square::F3,
 //!     promotion: None,
 //! });
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//!
+//! # #[derive(Debug)] struct CommonError;
+//! # impl From<ParseSanError> for CommonError { fn from(_: ParseSanError) -> Self { Self } }
+//! # impl From<SanError> for CommonError { fn from(_: SanError) -> Self { Self } }
+//! # Ok::<_, CommonError>(())
 //! ```
 //!
 //! Back to a (possibly disambiguated) SAN:
 //!
 //! ```
-//! # use shakmaty::{Chess, Position, Role, san::San};
+//! # use shakmaty::{Chess, Position, Role, san::{ParseSanError, San, SanError}};
 //! #
 //! # let pos = Chess::default();
 //! # let san: San = "Nf3".parse()?;
 //! # let m = san.to_move(&pos)?;
 //! assert_eq!(San::from_move(&pos, &m).to_string(), "Nf3");
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//!
+//! # #[derive(Debug)] struct CommonError;
+//! # impl From<ParseSanError> for CommonError { fn from(_: ParseSanError) -> Self { Self } }
+//! # impl From<SanError> for CommonError { fn from(_: SanError) -> Self { Self } }
+//! # Ok::<_, CommonError>(())
 //! ```
 
-use std::{error::Error, fmt, str::FromStr};
+use core::{fmt, str::FromStr};
 
 use crate::{CastlingSide, File, Move, MoveList, Outcome, Position, Rank, Role, Square};
 
@@ -74,7 +66,8 @@ impl fmt::Display for ParseSanError {
     }
 }
 
-impl Error for ParseSanError {}
+#[cfg(feature = "std")]
+impl std::error::Error for ParseSanError {}
 
 /// `IllegalSan` or `AmbiguousSan`.
 #[derive(Clone, Debug, Eq, PartialEq)]
@@ -94,7 +87,8 @@ impl fmt::Display for SanError {
     }
 }
 
-impl Error for SanError {}
+#[cfg(feature = "std")]
+impl std::error::Error for SanError {}
 
 /// A move in Standard Algebraic Notation.
 #[derive(Debug, PartialEq, Eq, Clone, Hash)]
@@ -145,16 +139,14 @@ impl San {
         } else {
             let mut chars = san.iter().copied();
 
-            let (role, next) = {
-                let ch = chars.next().ok_or(ParseSanError)?;
-                if ch >= b'a' {
-                    (Role::Pawn, ch)
-                } else {
-                    (
-                        Role::from_char(char::from(ch)).ok_or(ParseSanError)?,
-                        chars.next().ok_or(ParseSanError)?,
-                    )
-                }
+            let ch = chars.next().ok_or(ParseSanError)?;
+            let (role, next) = if ch.is_ascii_uppercase() {
+                (
+                    Role::from_char(char::from(ch)).ok_or(ParseSanError)?,
+                    chars.next().ok_or(ParseSanError)?,
+                )
+            } else {
+                (Role::Pawn, ch)
             };
 
             let (file, next) = if let Some(file) = File::from_char(char::from(next)) {
@@ -424,7 +416,8 @@ impl San {
     /// // other file does not match
     /// let nef3 = San::from_ascii(b"Nef3")?;
     /// assert!(!nef3.matches(&m));
-    /// # Ok::<_, Box<dyn std::error::Error>>(())
+    ///
+    /// # Ok::<_, shakmaty::san::ParseSanError>(())
     /// ```
     pub fn matches(&self, m: &Move) -> bool {
         match *self {
@@ -501,7 +494,7 @@ impl fmt::Display for San {
                 if capture {
                     write!(f, "x")?;
                 }
-                write!(f, "{}", to)?;
+                write!(f, "{to}")?;
                 if let Some(promotion) = promotion {
                     write!(f, "={}", promotion.upper_char())?;
                 }
@@ -512,7 +505,7 @@ impl fmt::Display for San {
             San::Put {
                 role: Role::Pawn,
                 to,
-            } => write!(f, "@{}", to),
+            } => write!(f, "@{to}"),
             San::Put { role, to } => write!(f, "{}@{}", role.upper_char(), to),
             San::Null => write!(f, "--"),
         }
@@ -527,14 +520,14 @@ pub enum Suffix {
 }
 
 impl Suffix {
-    pub fn char(self) -> char {
+    pub const fn char(self) -> char {
         match self {
             Suffix::Check => '+',
             Suffix::Checkmate => '#',
         }
     }
 
-    pub fn from_char(ch: char) -> Option<Suffix> {
+    pub const fn from_char(ch: char) -> Option<Suffix> {
         match ch {
             '+' => Some(Suffix::Check),
             '#' => Some(Suffix::Checkmate),
@@ -633,7 +626,7 @@ impl fmt::Display for SanPlus {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.san)?;
         if let Some(suffix) = self.suffix {
-            write!(f, "{}", suffix)?;
+            write!(f, "{suffix}")?;
         }
         Ok(())
     }
@@ -641,10 +634,20 @@ impl fmt::Display for SanPlus {
 
 #[cfg(test)]
 mod tests {
-    use std::mem;
+    #[cfg(feature = "alloc")]
+    use alloc::string::ToString;
+    use core::mem;
 
     use super::*;
+    use crate::{fen::Fen, CastlingMode, Chess};
 
+    #[test]
+    fn test_size() {
+        assert!(mem::size_of::<San>() <= 8);
+        assert!(mem::size_of::<SanPlus>() <= 8);
+    }
+
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_read_write() {
         for san in &[
@@ -658,8 +661,39 @@ mod tests {
     }
 
     #[test]
-    fn test_size() {
-        assert!(mem::size_of::<San>() <= 8);
-        assert!(mem::size_of::<SanPlus>() <= 8);
+    fn test_pawn_capture_without_file() {
+        let san = "f6".parse::<San>().expect("valid san");
+
+        let pos = "4k3/8/5p2/4P3/8/8/8/4K3 w -"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_position::<Chess>(CastlingMode::Standard)
+            .expect("legal fen");
+        assert_eq!(san.to_move(&pos), Err(SanError::IllegalSan));
+
+        let pos = "4k3/8/8/4Pp2/8/8/8/4K3 w - f6"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_position::<Chess>(CastlingMode::Standard)
+            .expect("legal fen");
+        assert_eq!(san.to_move(&pos), Err(SanError::IllegalSan));
+    }
+
+    #[cfg(feature = "alloc")]
+    #[test]
+    fn test_lax_pawn_move_san_roundtrip() {
+        let san = "6h8".parse::<San>().expect("kinda valid san");
+        assert_eq!(
+            san,
+            San::Normal {
+                role: Role::Pawn,
+                file: None,
+                rank: Some(Rank::Sixth),
+                capture: false,
+                to: Square::H8,
+                promotion: None,
+            }
+        );
+        assert_eq!(san.to_string(), "6h8");
     }
 }
