@@ -1,20 +1,4 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
-use std::num::NonZeroU32;
+use core::{convert::identity, num::NonZeroU32};
 
 use crate::{
     attacks, Bitboard, Board, ByColor, ByRole, CastlingMode, CastlingSide, Color, File, FromSetup,
@@ -22,6 +6,11 @@ use crate::{
 };
 
 /// A not necessarily legal position.
+///
+/// # Equality
+///
+/// [`Hash`](std::hash::Hash), [`PartialEq`](std::cmp::PartialEq), and
+/// [`Eq`](std::cmp::Eq) are implemented in terms of structural equality.
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct Setup {
     /// Piece positions on the board.
@@ -38,6 +27,8 @@ pub struct Setup {
 
     /// Castling rights in terms of corresponding rook positions.
     ///
+    /// # Examples
+    ///
     /// ```
     /// use shakmaty::{Bitboard, Setup};
     ///
@@ -92,7 +83,8 @@ pub struct Setup {
 }
 
 impl Setup {
-    pub fn empty() -> Setup {
+    /// Plain, empty board. No pieces. White to play.
+    pub const fn empty() -> Setup {
         Setup {
             board: Board::empty(),
             pockets: None,
@@ -102,7 +94,19 @@ impl Setup {
             ep_square: None,
             remaining_checks: None,
             halfmoves: 0,
-            fullmoves: NonZeroU32::new(1).unwrap(),
+            fullmoves: match NonZeroU32::new(1) {
+                Some(num) => num,
+                _ => unreachable!(),
+            },
+        }
+    }
+
+    /// Default board setup.
+    pub const fn initial() -> Setup {
+        Setup {
+            board: Board::new(),
+            castling_rights: Bitboard::CORNERS,
+            ..Setup::empty()
         }
     }
 
@@ -118,11 +122,7 @@ impl Setup {
 
 impl Default for Setup {
     fn default() -> Setup {
-        Setup {
-            board: Board::default(),
-            castling_rights: Bitboard::CORNERS,
-            ..Setup::empty()
-        }
+        Setup::initial()
     }
 }
 
@@ -130,63 +130,71 @@ impl Default for Setup {
 #[derive(Clone, Debug)]
 pub struct Castles {
     mask: Bitboard,
-    rook: [[Option<Square>; 2]; 2],
-    path: [[Bitboard; 2]; 2],
+    rook: ByColor<[Option<Square>; 2]>,
+    path: ByColor<[Bitboard; 2]>,
     mode: CastlingMode,
 }
 
+impl Castles {
+    pub const fn new() -> Castles {
+        Castles {
+            mode: CastlingMode::Standard,
+            mask: Bitboard::CORNERS,
+            rook: ByColor {
+                black: [Some(Square::H8), Some(Square::A8)],
+                white: [Some(Square::H1), Some(Square::A1)],
+            },
+            path: ByColor {
+                black: [
+                    Bitboard(0x6000_0000_0000_0000),
+                    Bitboard(0x0e00_0000_0000_0000),
+                ],
+                white: [
+                    Bitboard(0x0000_0000_0000_0060),
+                    Bitboard(0x0000_0000_0000_000e),
+                ],
+            },
+        }
+    }
+}
+
 impl Default for Castles {
     fn default() -> Castles {
-        Castles {
-            mode: CastlingMode::Standard,
-            mask: Bitboard::CORNERS,
-            rook: [
-                [Some(Square::H8), Some(Square::A8)], // black
-                [Some(Square::H1), Some(Square::A1)], // white
-            ],
-            path: [
-                [
-                    Bitboard(0x6000_0000_0000_0000),
-                    Bitboard(0x0e00_0000_0000_0000),
-                ],
-                [
-                    Bitboard(0x0000_0000_0000_0060),
-                    Bitboard(0x0000_0000_0000_000e),
-                ],
-            ],
-        }
+        Castles::new()
     }
 }
 
 impl CastlingMode {
     pub fn detect(setup: &Setup) -> CastlingMode {
-        let standard = Castles::from_setup(setup, CastlingMode::Standard).unwrap_or_else(|c| c);
-        let chess960 = Castles::from_setup(setup, CastlingMode::Chess960).unwrap_or_else(|c| c);
+        let standard = Castles::from_setup(setup, CastlingMode::Standard).unwrap_or_else(identity);
+        let chess960 = Castles::from_setup(setup, CastlingMode::Chess960).unwrap_or_else(identity);
         CastlingMode::from_standard(standard.mask == chess960.mask)
     }
 }
 
 impl Castles {
-    pub fn empty(mode: CastlingMode) -> Castles {
+    pub const fn empty(mode: CastlingMode) -> Castles {
         Castles {
             mode,
             mask: Bitboard(0),
-            rook: [[None; 2]; 2],
-            path: [[Bitboard(0); 2]; 2],
+            rook: ByColor {
+                black: [None; 2],
+                white: [None; 2],
+            },
+            path: ByColor {
+                black: [Bitboard::EMPTY; 2],
+                white: [Bitboard::EMPTY; 2],
+            },
         }
     }
 
     pub fn from_setup(setup: &Setup, mode: CastlingMode) -> Result<Castles, Castles> {
         let mut castles = Castles::empty(mode);
-
         let rooks = setup.castling_rights & setup.board.rooks();
 
         for color in Color::ALL {
             if let Some(king) = setup.board.king_of(color) {
-                if king.file() == File::A
-                    || king.file() == File::H
-                    || king.rank() != color.fold_wb(Rank::First, Rank::Eighth)
-                {
+                if king.rank() != color.fold_wb(Rank::First, Rank::Eighth) {
                     continue;
                 }
 
@@ -198,9 +206,9 @@ impl Castles {
                     let chess960 = king.file() != File::E || a_side.file() != File::A;
                     if !chess960 || mode.is_chess960() {
                         castles.mask.add(a_side);
-                        castles.rook[color as usize][CastlingSide::QueenSide as usize] =
+                        castles.rook.get_mut(color)[CastlingSide::QueenSide as usize] =
                             Some(a_side);
-                        castles.path[color as usize][CastlingSide::QueenSide as usize] =
+                        castles.path.get_mut(color)[CastlingSide::QueenSide as usize] =
                             (attacks::between(a_side, rook_to).with(rook_to)
                                 | attacks::between(king, king_to).with(king_to))
                             .without(king)
@@ -214,9 +222,8 @@ impl Castles {
                     let chess960 = king.file() != File::E || h_side.file() != File::H;
                     if !chess960 || mode.is_chess960() {
                         castles.mask.add(h_side);
-                        castles.rook[color as usize][CastlingSide::KingSide as usize] =
-                            Some(h_side);
-                        castles.path[color as usize][CastlingSide::KingSide as usize] =
+                        castles.rook.get_mut(color)[CastlingSide::KingSide as usize] = Some(h_side);
+                        castles.path.get_mut(color)[CastlingSide::KingSide as usize] =
                             (attacks::between(h_side, rook_to).with(rook_to)
                                 | attacks::between(king, king_to).with(king_to))
                             .without(king)
@@ -233,11 +240,11 @@ impl Castles {
         }
     }
 
-    pub fn any(&self) -> bool {
+    pub const fn any(&self) -> bool {
         self.mask.any()
     }
 
-    pub fn is_empty(&self) -> bool {
+    pub const fn is_empty(&self) -> bool {
         self.mask.is_empty()
     }
 
@@ -245,31 +252,43 @@ impl Castles {
         self.rook(color, side).is_some()
     }
 
-    pub fn has_side(&self, color: Color) -> bool {
-        (self.mask & color.backrank()).any()
+    pub const fn has_color(&self, color: Color) -> bool {
+        self.mask
+            .intersect(Bitboard::from_rank(color.backrank()))
+            .any()
     }
 
     pub fn discard_rook(&mut self, square: Square) {
-        if self.mask.remove(square) {
-            self.rook[0][0] = self.rook[0][0].filter(|sq| *sq != square);
-            self.rook[0][1] = self.rook[0][1].filter(|sq| *sq != square);
-            self.rook[1][0] = self.rook[1][0].filter(|sq| *sq != square);
-            self.rook[1][1] = self.rook[1][1].filter(|sq| *sq != square);
+        if square <= Square::H1 {
+            self.mask.discard(square);
+            if self.rook.white[0] == Some(square) {
+                self.rook.white[0] = None;
+            } else if self.rook.white[1] == Some(square) {
+                self.rook.white[1] = None;
+            }
+        } else if square >= Square::A8 {
+            self.mask.discard(square);
+            if self.rook.black[0] == Some(square) {
+                self.rook.black[0] = None;
+            } else if self.rook.black[1] == Some(square) {
+                self.rook.black[1] = None;
+            }
         }
     }
 
-    pub fn discard_side(&mut self, color: Color) {
+    pub fn discard_color(&mut self, color: Color) {
         self.mask.discard(color.backrank());
-        self.rook[color as usize] = [None, None];
+        *self.rook.get_mut(color) = [None, None];
     }
 
     #[inline]
-    pub fn rook(&self, color: Color, side: CastlingSide) -> Option<Square> {
-        self.rook[color as usize][side as usize]
+    pub const fn rook(&self, color: Color, side: CastlingSide) -> Option<Square> {
+        self.rook.get(color)[side as usize]
     }
 
     /// Gets the squares that need to be empty so that castling is possible
-    /// on the given side.
+    /// on the given side, assuming the player still has the required
+    /// castling rigths.
     ///
     /// # Examples
     ///
@@ -290,22 +309,23 @@ impl Castles {
     /// assert_eq!(path, Bitboard::from(Square::B1) | Bitboard::from(Square::C1) | Bitboard::from(Square::D1));
     /// ```
     #[inline]
-    pub fn path(&self, color: Color, side: CastlingSide) -> Bitboard {
-        self.path[color as usize][side as usize]
+    pub const fn path(&self, color: Color, side: CastlingSide) -> Bitboard {
+        self.path.get(color)[side as usize]
     }
 
+    /// Castling rigths in terms of corresponding rook positions.
     #[inline]
-    pub fn castling_rights(&self) -> Bitboard {
+    pub const fn castling_rights(&self) -> Bitboard {
         self.mask
     }
 
-    pub fn mode(&self) -> CastlingMode {
+    pub const fn mode(&self) -> CastlingMode {
         self.mode
     }
 }
 
 /// En passant square on the third or sixth rank.
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Copy, Clone)]
 pub(crate) struct EnPassant(pub Square);
 
 impl From<EnPassant> for Square {
@@ -345,7 +365,7 @@ impl EnPassant {
     }
 
     #[inline]
-    pub fn square(self) -> Square {
+    pub const fn square(self) -> Square {
         self.0
     }
 
