@@ -1,23 +1,6 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
-use std::{
+use core::{
     cmp::max,
     convert::TryInto,
-    error::Error,
     fmt::{self, Write as _},
     mem,
     num::TryFromIntError,
@@ -31,7 +14,6 @@ macro_rules! from_repr_u8_impl {
     ($from:ty, $($t:ty)+) => {
         $(impl From<$from> for $t {
             #[inline]
-            #[allow(clippy::cast_lossless)]
             fn from(value: $from) -> $t {
                 value as u8 as $t
             }
@@ -41,12 +23,11 @@ macro_rules! from_repr_u8_impl {
 
 macro_rules! try_from_int_impl {
     ($type:ty, $lower:expr, $upper:expr, $($t:ty)+) => {
-        $(impl std::convert::TryFrom<$t> for $type {
+        $(impl core::convert::TryFrom<$t> for $type {
             type Error = TryFromIntError;
 
             #[inline]
             #[allow(unused_comparisons)]
-            #[allow(clippy::cast_lossless)]
             fn try_from(value: $t) -> Result<$type, Self::Error> {
                 if ($lower..$upper).contains(&value) {
                     Ok(<$type>::new(value as u32))
@@ -58,47 +39,6 @@ macro_rules! try_from_int_impl {
     }
 }
 
-macro_rules! unsafe_step_impl {
-    ($type:ty) => {
-        #[cfg(feature = "step")]
-        impl std::iter::Step for $type {
-            fn steps_between(start: &Self, end: &Self) -> Option<usize> {
-                usize::from(*end).checked_sub(usize::from(*start))
-            }
-
-            fn forward_checked(start: Self, count: usize) -> Option<Self> {
-                use std::convert::TryFrom as _;
-                i32::try_from(count)
-                    .ok()
-                    .and_then(|count| start.offset(count))
-            }
-
-            fn backward_checked(start: Self, count: usize) -> Option<Self> {
-                use std::convert::TryFrom as _;
-                i32::try_from(count)
-                    .ok()
-                    .and_then(|count| start.offset(-count))
-            }
-
-            unsafe fn forward_unchecked(start: Self, count: usize) -> Self {
-                // Safety: It is the callers responsibility to ensure
-                // start + count is in the range of Self.
-                debug_assert!(count < 64);
-                let count = count as u32;
-                unsafe { Self::new_unchecked(u32::from(start) + count) }
-            }
-
-            unsafe fn backward_unchecked(start: Self, count: usize) -> Self {
-                // Safety: It is the callers responsibility to ensure
-                // start - count is in the range of Self.
-                debug_assert!(count < 64);
-                let count = count as u32;
-                unsafe { Self::new_unchecked(u32::from(start) - count) }
-            }
-        }
-    };
-}
-
 /// A file of the chessboard.
 #[allow(missing_docs)]
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
@@ -120,8 +60,9 @@ impl File {
     /// # Panics
     ///
     /// Panics if the index is not in the range `0..=7`.
+    #[track_caller]
     #[inline]
-    pub fn new(index: u32) -> File {
+    pub const fn new(index: u32) -> File {
         assert!(index < 8);
         unsafe { File::new_unchecked(index) }
     }
@@ -133,7 +74,7 @@ impl File {
     /// It is the callers responsibility to ensure the index is in the range
     /// `0..=7`.
     #[inline]
-    pub unsafe fn new_unchecked(index: u32) -> File {
+    pub const unsafe fn new_unchecked(index: u32) -> File {
         debug_assert!(index < 8);
         unsafe { mem::transmute(index as u8) }
     }
@@ -165,6 +106,11 @@ impl File {
             .and_then(|index| index.try_into().ok())
     }
 
+    #[inline]
+    pub fn distance(self, other: File) -> u32 {
+        u32::from(self).abs_diff(u32::from(other))
+    }
+
     #[must_use]
     #[inline]
     pub fn flip_horizontal(self) -> File {
@@ -213,7 +159,6 @@ impl fmt::Display for File {
 
 from_repr_u8_impl! { File, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
 try_from_int_impl! { File, 0, 8, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
-unsafe_step_impl! { File }
 
 /// A rank of the chessboard.
 #[allow(missing_docs)]
@@ -236,6 +181,7 @@ impl Rank {
     /// # Panics
     ///
     /// Panics if the index is not in the range `0..=7`.
+    #[track_caller]
     #[inline]
     pub fn new(index: u32) -> Rank {
         assert!(index < 8);
@@ -276,6 +222,11 @@ impl Rank {
             .and_then(|index| index.try_into().ok())
     }
 
+    #[inline]
+    pub fn distance(self, other: Rank) -> u32 {
+        u32::from(self).abs_diff(u32::from(other))
+    }
+
     #[must_use]
     #[inline]
     pub fn flip_vertical(self) -> Rank {
@@ -324,7 +275,6 @@ impl fmt::Display for Rank {
 
 from_repr_u8_impl! { Rank, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
 try_from_int_impl! { Rank, 0, 8, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
-unsafe_step_impl! { Rank }
 
 /// Error when parsing an invalid square name.
 #[derive(Clone, Debug)]
@@ -336,7 +286,8 @@ impl fmt::Display for ParseSquareError {
     }
 }
 
-impl Error for ParseSquareError {}
+#[cfg(feature = "std")]
+impl std::error::Error for ParseSquareError {}
 
 /// A square of the chessboard.
 #[rustfmt::skip]
@@ -369,8 +320,9 @@ impl Square {
     /// assert_eq!(Square::new(0), Square::A1);
     /// assert_eq!(Square::new(63), Square::H8);
     /// ```
+    #[track_caller]
     #[inline]
-    pub fn new(index: u32) -> Square {
+    pub const fn new(index: u32) -> Square {
         assert!(index < 64);
         unsafe { Square::new_unchecked(index) }
     }
@@ -381,7 +333,7 @@ impl Square {
     ///
     /// It is the callers responsibility to ensure it is in the range `0..=63`.
     #[inline]
-    pub unsafe fn new_unchecked(index: u32) -> Square {
+    pub const unsafe fn new_unchecked(index: u32) -> Square {
         debug_assert!(index < 64);
         unsafe { mem::transmute(index as u8) }
     }
@@ -412,14 +364,10 @@ impl Square {
     /// # Example
     ///
     /// ```
-    /// # use std::error::Error;
-    /// #
     /// use shakmaty::Square;
-    ///
     /// let sq = Square::from_ascii(b"a5")?;
     /// assert_eq!(sq, Square::A5);
-    /// #
-    /// # Ok::<_, Box<dyn Error>>(())
+    /// # Ok::<_, shakmaty::ParseSquareError>(())
     /// ```
     #[inline]
     pub fn from_ascii(s: &[u8]) -> Result<Square, ParseSquareError> {
@@ -534,7 +482,6 @@ impl Square {
     /// ```
     #[must_use]
     #[inline]
-    #[allow(clippy::unusual_byte_groupings)]
     pub fn flip_horizontal(self) -> Square {
         self.xor(Square::H1)
     }
@@ -549,7 +496,6 @@ impl Square {
     /// ```
     #[must_use]
     #[inline]
-    #[allow(clippy::unusual_byte_groupings)]
     pub fn flip_vertical(self) -> Square {
         self.xor(Square::A8)
     }
@@ -609,7 +555,6 @@ impl Square {
     /// ```
     #[must_use]
     #[inline]
-    #[allow(clippy::unusual_byte_groupings)]
     pub fn rotate_180(self) -> Square {
         self.xor(Square::H8)
     }
@@ -664,8 +609,8 @@ impl Square {
     /// ```
     pub fn distance(self, other: Square) -> u32 {
         max(
-            (self.file() - other.file()).unsigned_abs(),
-            (self.rank() - other.rank()).unsigned_abs(),
+            self.file().distance(other.file()),
+            self.rank().distance(other.rank()),
         )
     }
 }
@@ -690,7 +635,6 @@ mod all_squares {
 
 from_repr_u8_impl! { Square, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
 try_from_int_impl! { Square, 0, 64, u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
-unsafe_step_impl! { Square }
 
 impl Sub for Square {
     type Output = i32;
@@ -744,4 +688,14 @@ mod tests {
             }
         }
     }
+
+    #[cfg(feature = "nohash-hasher")]
+    #[test]
+    fn test_nohash_hasher() {
+        use core::hash::{Hash, Hasher};
+
+        let mut hasher = nohash_hasher::NoHashHasher::<Square>::default();
+        Square::H1.hash(&mut hasher);
+        assert_eq!(hasher.finish(), 7);
+    }
 }
