@@ -1,19 +1,3 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Parse and write moves in Universal Chess Interface representation.
 //!
 //! # Examples
@@ -30,13 +14,14 @@
 //!     to: Square::F3,
 //!     promotion: None,
 //! });
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//!
+//! # Ok::<_, shakmaty::uci::ParseUciError>(())
 //! ```
 //!
 //! Converting to a legal move in the context of a position:
 //!
 //! ```
-//! # use shakmaty::{Square, uci::Uci};
+//! # use shakmaty::{Square, uci::{IllegalUciError, ParseUciError, Uci}};
 //! use shakmaty::{Color::White, Chess, Setup, Position};
 //!
 //! # let uci: Uci = "g1f3".parse()?;
@@ -45,7 +30,11 @@
 //!
 //! pos.play_unchecked(&m);
 //! assert_eq!(pos.board().piece_at(Square::F3), Some(White.knight()));
-//! # Ok::<_, Box<dyn std::error::Error>>(())
+//!
+//! # #[derive(Debug)] struct CommonError;
+//! # impl From<IllegalUciError> for CommonError { fn from(_: IllegalUciError) -> Self { Self } }
+//! # impl From<ParseUciError> for CommonError { fn from(_: ParseUciError) -> Self { Self } }
+//! # Ok::<_, CommonError>(())
 //! ```
 //!
 //! Converting from [`Move`] to [`Uci`]:
@@ -75,7 +64,7 @@
 //!
 //! [`Move`]: super::Move
 
-use std::{error::Error, fmt, str::FromStr};
+use core::{fmt, str::FromStr};
 
 use crate::{CastlingMode, CastlingSide, Move, Position, Rank, Role, Square};
 
@@ -89,7 +78,8 @@ impl fmt::Display for ParseUciError {
     }
 }
 
-impl Error for ParseUciError {}
+#[cfg(feature = "std")]
+impl std::error::Error for ParseUciError {}
 
 /// Error when UCI is illegal.
 #[derive(Clone, Debug)]
@@ -101,7 +91,8 @@ impl fmt::Display for IllegalUciError {
     }
 }
 
-impl Error for IllegalUciError {}
+#[cfg(feature = "std")]
+impl std::error::Error for IllegalUciError {}
 
 /// A move as represented in the UCI protocol.
 #[derive(Clone, Eq, PartialEq, Debug, Hash)]
@@ -133,7 +124,7 @@ impl fmt::Display for Uci {
                 from,
                 to,
                 promotion: None,
-            } => write!(f, "{}{}", from, to),
+            } => write!(f, "{from}{to}"),
             Uci::Normal {
                 from,
                 to,
@@ -164,7 +155,8 @@ impl Uci {
     ///     to: Square::E5,
     ///     promotion: None,
     /// });
-    /// # Ok::<_, Box<dyn std::error::Error>>(())
+    ///
+    /// # Ok::<_, shakmaty::uci::ParseUciError>(())
     /// ```
     pub fn from_ascii(uci: &[u8]) -> Result<Uci, ParseUciError> {
         if uci.len() != 4 && uci.len() != 5 {
@@ -251,7 +243,7 @@ impl Uci {
     /// let uci = Uci::from_chess960(&m);
     /// assert_eq!(uci.to_string(), "e8h8");
     /// ```
-    pub fn from_chess960(m: &Move) -> Uci {
+    pub const fn from_chess960(m: &Move) -> Uci {
         match *m {
             Move::Normal {
                 from,
@@ -364,7 +356,7 @@ impl Move {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{fen::Fen, Chess, EnPassantMode};
+    use crate::{fen::Fen, Chess};
 
     #[test]
     pub fn test_uci_to_en_passant() {
@@ -461,8 +453,10 @@ mod tests {
         );
     }
 
+    #[cfg(feature = "alloc")]
     #[test]
     fn test_uci_to_castles() {
+        use alloc::string::ToString as _;
         let mut pos: Chess = "nbqrknbr/pppppppp/8/8/8/8/PPPPPPPP/NBQRKNBR w KQkq - 0 1"
             .parse::<Fen>()
             .expect("valid fen")
@@ -477,7 +471,7 @@ mod tests {
             pos.play_unchecked(&m);
         }
         assert_eq!(
-            Fen::from_position(pos, EnPassantMode::Legal).to_string(),
+            Fen::from_position(pos, crate::EnPassantMode::Legal).to_string(),
             "nbkr1nbr/ppp1pppp/3p4/8/5Pq1/6N1/PPPPPBPP/NBQR1RK1 b - - 5 4"
         );
     }
