@@ -1,19 +1,3 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 //! Zobrist hashing for positions.
 //!
 //! # Stability
@@ -22,21 +6,32 @@
 //! hash values is considered a semver breaking change and will be noted in the
 //! changelog.
 //!
+//! # Warning: Forged collisions
+//!
+//! Zobrist hashes have good collision resistance, but can be
+//! forged efficiently.
+//!
+//! Additionally, in this implementation, impossible positions with
+//! more than standard material in Crazyhouse pockets are particularly
+//! prone to collisions.
+//!
 //! # Examples
 //!
 //! ```
-//! use shakmaty::{Chess, zobrist::ZobristHash};
+//! use shakmaty::{Chess, EnPassantMode, zobrist::{Zobrist64, ZobristHash}};
 //!
 //! let pos = Chess::default();
-//! assert_eq!(pos.zobrist_hash::<u64>(), 0x463b96181691fc9c);
+//! assert_eq!(pos.zobrist_hash::<Zobrist64>(EnPassantMode::Legal), Zobrist64(0x463b96181691fc9c));
 //! ```
 
-use std::{cell::Cell, num::NonZeroU32, ops::BitXorAssign};
+use core::{
+    fmt,
+    hash::{Hash, Hasher},
+    ops::{BitXor, BitXorAssign},
+};
 
 use crate::{
-    color::ByColor, Bitboard, Board, ByRole, Castles, CastlingMode, CastlingSide, Chess, Color,
-    EnPassantMode, File, FromSetup, Move, MoveList, Outcome, Piece, Position, PositionError,
-    RemainingChecks, Role, Setup, Square,
+    Board, CastlingSide, Color, EnPassantMode, File, Piece, Position, RemainingChecks, Role, Square,
 };
 
 /// Integer type that can be returned as a Zobrist hash.
@@ -51,396 +46,370 @@ pub trait ZobristValue: BitXorAssign + Default + Copy {
 }
 
 macro_rules! zobrist_value_impl {
-    ($($t:ty)+) => {
-        $(impl ZobristValue for $t {
+    ($t:ident, $proxy:ty, $bits:literal) => {
+        #[derive(Default, Copy, Clone, Eq)]
+        #[doc = "A [`ZobristValue`] with "]
+        #[doc = stringify!($bits)]
+        #[doc = " bits."]
+        pub struct $t(pub $proxy);
+
+        impl PartialEq for $t {
+            #[inline]
+            fn eq(&self, other: &$t) -> bool {
+                self.0 == other.0
+            }
+        }
+
+        impl From<$proxy> for $t {
+            #[inline]
+            fn from(value: $proxy) -> $t {
+                $t(value)
+            }
+        }
+        impl From<$t> for $proxy {
+            #[inline]
+            fn from(value: $t) -> $proxy {
+                value.0
+            }
+        }
+
+        impl BitXor for $t {
+            type Output = $t;
+            #[inline]
+            fn bitxor(self, other: $t) -> $t {
+                $t(self.0 ^ other.0)
+            }
+        }
+        impl BitXor<&$t> for $t {
+            type Output = $t;
+            #[inline]
+            fn bitxor(self, other: &$t) -> $t {
+                $t(self.0 ^ other.0)
+            }
+        }
+        impl BitXor<&$t> for &$t {
+            type Output = $t;
+            #[inline]
+            fn bitxor(self, other: &$t) -> $t {
+                $t(self.0 ^ other.0)
+            }
+        }
+        impl BitXor<$t> for &$t {
+            type Output = $t;
+            #[inline]
+            fn bitxor(self, other: $t) -> $t {
+                $t(self.0 ^ other.0)
+            }
+        }
+        impl BitXorAssign for $t {
+            #[inline]
+            fn bitxor_assign(&mut self, rhs: $t) {
+                self.0 ^= rhs.0;
+            }
+        }
+        impl BitXorAssign<&$t> for $t {
+            #[inline]
+            fn bitxor_assign(&mut self, rhs: &$t) {
+                self.0 ^= rhs.0;
+            }
+        }
+
+        impl fmt::Debug for $t {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, concat!(stringify!($t), "({:#x})"), self.0)
+            }
+        }
+        impl fmt::UpperHex for $t {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                fmt::UpperHex::fmt(&self.0, f)
+            }
+        }
+        impl fmt::LowerHex for $t {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                fmt::LowerHex::fmt(&self.0, f)
+            }
+        }
+        impl fmt::Octal for $t {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                fmt::Octal::fmt(&self.0, f)
+            }
+        }
+        impl fmt::Binary for $t {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                fmt::Binary::fmt(&self.0, f)
+            }
+        }
+
+        impl ZobristValue for $t {
+            #[inline]
             fn zobrist_for_piece(square: Square, piece: Piece) -> $t {
                 let piece_idx = (usize::from(piece.role) - 1) * 2 + piece.color as usize;
-                PIECE_MASKS[64 * piece_idx + usize::from(square)] as $t
+                $t(PIECE_MASKS[64 * piece_idx + usize::from(square)] as $proxy)
             }
 
+            #[inline]
             fn zobrist_for_white_turn() -> $t {
-                WHITE_TURN_MASK as $t
+                $t(WHITE_TURN_MASK as $proxy)
             }
 
+            #[inline]
             fn zobrist_for_castling_right(color: Color, side: CastlingSide) -> $t {
-                CASTLING_RIGHT_MASKS[match (color, side) {
+                $t(CASTLING_RIGHT_MASKS[match (color, side) {
                     (Color::White, CastlingSide::KingSide) => 0,
                     (Color::White, CastlingSide::QueenSide) => 1,
                     (Color::Black, CastlingSide::KingSide) => 2,
                     (Color::Black, CastlingSide::QueenSide) => 3,
-                }] as $t
+                }] as $proxy)
             }
 
+            #[inline]
             fn zobrist_for_en_passant_file(file: File) -> $t {
-                EN_PASSANT_FILE_MASKS[usize::from(file)] as $t
+                $t(EN_PASSANT_FILE_MASKS[usize::from(file)] as $proxy)
             }
 
+            #[inline]
             fn zobrist_for_remaining_checks(color: Color, remaining: RemainingChecks) -> $t {
                 if remaining < RemainingChecks::default() {
-                    REMAINING_CHECKS_MASKS[usize::from(remaining) + color.fold_wb(0, 3)] as $t
+                    $t(
+                        REMAINING_CHECKS_MASKS[usize::from(remaining) + color.fold_wb(0, 3)]
+                            as $proxy,
+                    )
                 } else {
                     <$t>::default()
                 }
             }
 
+            #[inline]
             fn zobrist_for_promoted(square: Square) -> $t {
-                PROMOTED_MASKS[usize::from(square)] as $t
+                $t(PROMOTED_MASKS[usize::from(square)] as $proxy)
             }
 
+            #[inline]
             fn zobrist_for_pocket(color: Color, role: Role, pieces: u8) -> $t {
                 if 0 < pieces && pieces <= 16 {
                     let color_idx = color as usize;
                     let role_idx = usize::from(role) - 1;
                     let pieces_idx = usize::from(pieces) - 1;
-                    POCKET_MASKS[color_idx * 6 * 16 + role_idx * 16 + pieces_idx] as $t
+                    $t(POCKET_MASKS[color_idx * 6 * 16 + role_idx * 16 + pieces_idx] as $proxy)
                 } else {
                     <$t>::default()
                 }
             }
-        })+
+        }
+
+        #[cfg(feature = "nohash-hasher")]
+        impl nohash_hasher::IsEnabled for $t {}
+    };
+}
+
+zobrist_value_impl! { Zobrist8, u8, 8 }
+zobrist_value_impl! { Zobrist16, u16, 16 }
+zobrist_value_impl! { Zobrist32, u32, 32 }
+zobrist_value_impl! { Zobrist64, u64, 64 }
+zobrist_value_impl! { Zobrist128, u128, 128 }
+
+impl Hash for Zobrist128 {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u64(self.0 as u64); // Truncating!
+    }
+}
+impl Hash for Zobrist64 {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u64(self.0);
+    }
+}
+impl Hash for Zobrist32 {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u32(self.0);
+    }
+}
+impl Hash for Zobrist16 {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u16(self.0);
+    }
+}
+impl Hash for Zobrist8 {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u8(self.0);
     }
 }
 
-zobrist_value_impl! { u8 u16 u32 u64 u128 }
+impl From<Zobrist128> for Zobrist64 {
+    fn from(value: Zobrist128) -> Zobrist64 {
+        Zobrist64(value.0 as u64)
+    }
+}
+impl From<Zobrist128> for Zobrist32 {
+    fn from(value: Zobrist128) -> Zobrist32 {
+        Zobrist32(value.0 as u32)
+    }
+}
+impl From<Zobrist128> for Zobrist16 {
+    fn from(value: Zobrist128) -> Zobrist16 {
+        Zobrist16(value.0 as u16)
+    }
+}
+impl From<Zobrist128> for Zobrist8 {
+    fn from(value: Zobrist128) -> Zobrist8 {
+        Zobrist8(value.0 as u8)
+    }
+}
+
+impl From<Zobrist64> for Zobrist32 {
+    fn from(value: Zobrist64) -> Zobrist32 {
+        Zobrist32(value.0 as u32)
+    }
+}
+impl From<Zobrist64> for Zobrist16 {
+    fn from(value: Zobrist64) -> Zobrist16 {
+        Zobrist16(value.0 as u16)
+    }
+}
+impl From<Zobrist64> for Zobrist8 {
+    fn from(value: Zobrist64) -> Zobrist8 {
+        Zobrist8(value.0 as u8)
+    }
+}
+
+impl From<Zobrist32> for Zobrist16 {
+    fn from(value: Zobrist32) -> Zobrist16 {
+        Zobrist16(value.0 as u16)
+    }
+}
+impl From<Zobrist32> for Zobrist8 {
+    fn from(value: Zobrist32) -> Zobrist8 {
+        Zobrist8(value.0 as u8)
+    }
+}
+
+impl From<Zobrist16> for Zobrist8 {
+    fn from(value: Zobrist16) -> Zobrist8 {
+        Zobrist8(value.0 as u8)
+    }
+}
 
 /// Supports Zobrist hashing.
 pub trait ZobristHash {
-    /// Computes the Zobrist hash of the position from scratch. Hash includes
-    /// the position, except halfmove clock and fullmove number.
-    ///
-    /// Warning: Zobrist hashes have good collision resistance, but can be
-    /// forged efficiently. In this implementation, impossible positions with
-    /// more than 3 remaining checks in Three-check and more than standard
-    /// material in Crazyhouse pockets are particularly prone to collisions.
-    fn zobrist_hash<V: ZobristValue>(&self) -> V;
-
-    /// Prepares an incremental update of the Zobrist hash before playing move
-    /// `m` in `self`. Returns a new intermediate Zobrist hash, or `None`
-    /// if incremental updating is not supported.
-    fn prepare_incremental_zobrist_hash<V: ZobristValue>(
-        &self,
-        _previous: V,
-        _m: &Move,
-    ) -> Option<V> {
-        None
-    }
-
-    /// Finalizes an incremental update of the Zobrist hash after playing move
-    /// `m` in `self`. Returns the new Zobrist hash, or `None` if incremental
-    /// updating is not supported.
-    fn finalize_incremental_zobrist_hash<V: ZobristValue>(
-        &self,
-        _intermediate: V,
-        _m: &Move,
-    ) -> Option<V> {
-        None
-    }
+    /// Computes the Zobrist hash of the position from scratch. The hash
+    /// includes the position, except halfmove clock and fullmove number.
+    fn zobrist_hash<V: ZobristValue>(&self, mode: EnPassantMode) -> V;
 }
 
-impl ZobristHash for Chess {
-    fn zobrist_hash<V: ZobristValue>(&self) -> V {
-        hash_position(self)
-    }
-}
+impl<P: Position> ZobristHash for P {
+    fn zobrist_hash<V: ZobristValue>(&self, mode: EnPassantMode) -> V {
+        let mut zobrist = hash_board(self.board());
 
-#[cfg(feature = "variant")]
-mod variant {
-    use super::*;
-
-    impl ZobristHash for crate::variant::Antichess {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        for sq in self.promoted() {
+            zobrist ^= V::zobrist_for_promoted(sq);
         }
-    }
 
-    impl ZobristHash for crate::variant::Atomic {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        if let Some(pockets) = self.pockets() {
+            for (color, pocket) in pockets.as_ref().zip_color() {
+                for role in Role::ALL {
+                    zobrist ^= V::zobrist_for_pocket(color, role, *pocket.get(role));
+                }
+            }
         }
-    }
 
-    impl ZobristHash for crate::variant::Crazyhouse {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        if self.turn() == Color::White {
+            zobrist ^= V::zobrist_for_white_turn();
         }
-    }
 
-    impl ZobristHash for crate::variant::Horde {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        let castles = self.castles();
+        for color in Color::ALL {
+            for side in CastlingSide::ALL {
+                if castles.has(color, side) {
+                    zobrist ^= V::zobrist_for_castling_right(color, side);
+                }
+            }
         }
-    }
 
-    impl ZobristHash for crate::variant::KingOfTheHill {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        if let Some(sq) = self.ep_square(mode) {
+            zobrist ^= V::zobrist_for_en_passant_file(sq.file());
         }
-    }
 
-    impl ZobristHash for crate::variant::RacingKings {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
+        if let Some(remaining_checks) = self.remaining_checks() {
+            for (color, remaining) in remaining_checks.as_ref().zip_color() {
+                zobrist ^= V::zobrist_for_remaining_checks(color, *remaining);
+            }
         }
-    }
 
-    impl ZobristHash for crate::variant::ThreeCheck {
-        fn zobrist_hash<V: ZobristValue>(&self) -> V {
-            hash_position(self)
-        }
-    }
-}
-
-/// A wrapper for [`Position`] that maintains an incremental Zobrist hash.
-///
-/// # Examples
-///
-/// ```
-/// use shakmaty::{Chess, Position, Move, Square, Role, zobrist::Zobrist};
-///
-/// let pos: Zobrist<Chess, u64> = Zobrist::default();
-///
-/// assert_eq!(pos.zobrist_hash(), 0x463b96181691fc9c);
-/// assert_eq!(pos.zobrist_hash(), 0x463b96181691fc9c); // cached
-///
-/// // 1. e4
-/// let pos = pos.play(&Move::Normal {
-///     role: Role::Pawn,
-///     from: Square::E2,
-///     to: Square::E4,
-///     capture: None,
-///     promotion: None,
-/// })?;
-///
-/// // Incrementally updated (or recomputed from scratch if incremental
-/// // updates not supported).
-/// assert_eq!(pos.zobrist_hash(), 0x823c9b50fd114196);
-/// # Ok::<_, Box<dyn std::error::Error>>(())
-/// ```
-#[derive(Debug, Clone)]
-pub struct Zobrist<P, V: ZobristValue> {
-    pos: P,
-    zobrist: Cell<Option<V>>,
-}
-
-impl<P, V: ZobristValue> Zobrist<P, V> {
-    pub fn new(pos: P) -> Zobrist<P, V> {
-        Zobrist {
-            pos,
-            zobrist: Cell::new(None),
-        }
-    }
-
-    pub fn into_inner(self) -> P {
-        self.pos
-    }
-
-    pub fn as_inner(&self) -> &P {
-        &self.pos
-    }
-}
-
-impl<P: ZobristHash, V: ZobristValue> Zobrist<P, V> {
-    pub fn zobrist_hash(&self) -> V {
-        if let Some(zobrist) = self.zobrist.get() {
-            zobrist
-        } else {
-            let zobrist = self.pos.zobrist_hash();
-            self.zobrist.set(Some(zobrist));
-            zobrist
-        }
-    }
-}
-
-impl<P: Default, V: ZobristValue> Default for Zobrist<P, V> {
-    fn default() -> Zobrist<P, V> {
-        Self::new(P::default())
-    }
-}
-
-impl<P: FromSetup + Position, V: ZobristValue> FromSetup for Zobrist<P, V> {
-    fn from_setup(setup: Setup, mode: CastlingMode) -> Result<Self, PositionError<Self>> {
-        match P::from_setup(setup, mode) {
-            Ok(pos) => Ok(Zobrist::new(pos)),
-            Err(err) => Err(PositionError {
-                pos: Zobrist::new(err.pos),
-                errors: err.errors,
-            }),
-        }
-    }
-}
-
-impl<P: Position + ZobristHash, V: ZobristValue> Position for Zobrist<P, V> {
-    fn board(&self) -> &Board {
-        self.pos.board()
-    }
-    fn promoted(&self) -> Bitboard {
-        self.pos.promoted()
-    }
-    fn pockets(&self) -> Option<&ByColor<ByRole<u8>>> {
-        self.pos.pockets()
-    }
-    fn turn(&self) -> Color {
-        self.pos.turn()
-    }
-    fn castles(&self) -> &Castles {
-        self.pos.castles()
-    }
-    fn maybe_ep_square(&self) -> Option<Square> {
-        self.pos.maybe_ep_square()
-    }
-    fn remaining_checks(&self) -> Option<&ByColor<RemainingChecks>> {
-        self.pos.remaining_checks()
-    }
-    fn halfmoves(&self) -> u32 {
-        self.pos.halfmoves()
-    }
-    fn fullmoves(&self) -> NonZeroU32 {
-        self.pos.fullmoves()
-    }
-    fn into_setup(self, mode: EnPassantMode) -> Setup {
-        self.pos.into_setup(mode)
-    }
-    fn legal_moves(&self) -> MoveList {
-        self.pos.legal_moves()
-    }
-    fn san_candidates(&self, role: Role, to: Square) -> MoveList {
-        self.pos.san_candidates(role, to)
-    }
-    fn castling_moves(&self, side: CastlingSide) -> MoveList {
-        self.pos.castling_moves(side)
-    }
-    fn en_passant_moves(&self) -> MoveList {
-        self.pos.en_passant_moves()
-    }
-    fn capture_moves(&self) -> MoveList {
-        self.pos.capture_moves()
-    }
-    fn promotion_moves(&self) -> MoveList {
-        self.pos.promotion_moves()
-    }
-    fn is_irreversible(&self, m: &Move) -> bool {
-        self.pos.is_irreversible(m)
-    }
-    fn king_attackers(&self, square: Square, attacker: Color, occupied: Bitboard) -> Bitboard {
-        self.pos.king_attackers(square, attacker, occupied)
-    }
-    fn is_variant_end(&self) -> bool {
-        self.pos.is_variant_end()
-    }
-    fn has_insufficient_material(&self, color: Color) -> bool {
-        self.pos.has_insufficient_material(color)
-    }
-    fn variant_outcome(&self) -> Option<Outcome> {
-        self.pos.variant_outcome()
-    }
-
-    fn play_unchecked(&mut self, m: &Move) {
-        self.zobrist.set(
-            self.zobrist
-                .get()
-                .and_then(|value| self.pos.prepare_incremental_zobrist_hash(value, m)),
-        );
-        self.pos.play_unchecked(m);
-        self.zobrist.set(
-            self.zobrist
-                .get()
-                .and_then(|value| self.pos.finalize_incremental_zobrist_hash(value, m)),
-        );
+        zobrist
     }
 }
 
 fn hash_board<V: ZobristValue>(board: &Board) -> V {
+    // Order optimized for cache efficiency.
     let mut zobrist = V::default();
-    for (sq, piece) in board.clone() {
-        zobrist ^= V::zobrist_for_piece(sq, piece);
-    }
-    zobrist
-}
-
-fn hash_position<P: Position, V: ZobristValue>(pos: &P) -> V {
-    let mut zobrist = hash_board(pos.board());
-
-    for sq in pos.promoted() {
-        zobrist ^= V::zobrist_for_promoted(sq);
-    }
-
-    if let Some(pockets) = pos.pockets() {
-        for (color, pocket) in pockets.as_ref().zip_color() {
-            for role in Role::ALL {
-                zobrist ^= V::zobrist_for_pocket(color, role, *pocket.get(role));
+    for role in Role::ALL {
+        for color in [Color::Black, Color::White] {
+            let piece = role.of(color);
+            for sq in board.by_piece(piece) {
+                zobrist ^= V::zobrist_for_piece(sq, piece);
             }
         }
     }
-
-    if pos.turn() == Color::White {
-        zobrist ^= V::zobrist_for_white_turn();
-    }
-
-    let castles = pos.castles();
-    for color in Color::ALL {
-        for side in CastlingSide::ALL {
-            if castles.has(color, side) {
-                zobrist ^= V::zobrist_for_castling_right(color, side);
-            }
-        }
-    }
-
-    if let Some(sq) = pos.legal_ep_square() {
-        zobrist ^= V::zobrist_for_en_passant_file(sq.file());
-    }
-
-    if let Some(remaining_checks) = pos.remaining_checks() {
-        for (color, remaining) in remaining_checks.as_ref().zip_color() {
-            zobrist ^= V::zobrist_for_remaining_checks(color, *remaining);
-        }
-    }
-
     zobrist
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{fen::Fen, uci::Uci, Chess};
+    use crate::{fen::Fen, CastlingMode, Chess};
 
     #[test]
     fn test_polyglot() {
         let reference_values = [
             (
                 "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1",
-                0x463b96181691fc9c,
+                Zobrist64(0x463b96181691fc9c),
             ),
             (
                 "rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq - 0 1",
-                0x823c9b50fd114196,
+                Zobrist64(0x823c9b50fd114196),
             ),
             (
                 "rnbqkbnr/ppp1pppp/8/3p4/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2",
-                0x0756b94461c50fb0,
+                Zobrist64(0x0756b94461c50fb0),
             ),
             (
                 "rnbqkbnr/ppp1pppp/8/3pP3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2",
-                0x662fafb965db29d4,
+                Zobrist64(0x662fafb965db29d4),
             ),
             (
                 "rnbqkbnr/ppp1p1pp/8/3pPp2/8/8/PPPP1PPP/RNBQKBNR w KQkq f6 0 3",
-                0x22a48b5a8e47ff78,
+                Zobrist64(0x22a48b5a8e47ff78),
             ),
             (
                 "rnbqkbnr/ppp1p1pp/8/3pPp2/8/8/PPPPKPPP/RNBQ1BNR b kq - 1 3",
-                0x652a607ca3f242c1,
+                Zobrist64(0x652a607ca3f242c1),
             ),
             (
                 "rnbq1bnr/ppp1pkpp/8/3pPp2/8/8/PPPPKPPP/RNBQ1BNR w - - 2 4",
-                0x00fdd303c946bdd9,
+                Zobrist64(0x00fdd303c946bdd9),
             ),
             (
                 "rnbqkbnr/p1pppppp/8/8/PpP4P/8/1P1PPPP1/RNBQKBNR b KQkq c3 0 3",
-                0x3c8123ea7b067637,
+                Zobrist64(0x3c8123ea7b067637),
             ),
             (
                 "rnbqkbnr/p1pppppp/8/8/P6P/R1p5/1P1PPPP1/1NBQKBNR b Kkq - 1 4",
-                0x5c3f9b829b279560,
+                Zobrist64(0x5c3f9b829b279560),
             ),
         ];
 
@@ -451,38 +420,12 @@ mod tests {
                 .into_position(CastlingMode::Standard)
                 .expect("legal position");
 
-            assert_eq!(pos.zobrist_hash::<u64>(), expected, "{}", fen);
-        }
-    }
-
-    #[test]
-    fn test_incremental() {
-        let moves = [
-            "e2e4", "e7e5", "g2g3", "g8f6", "f1g2", "f8c5", "g1e2", "h7h5", "h2h3", "h5h4", "g3g4",
-            "d7d5", "e4d5", "f6d5", "d2d4", "e5d4", "e2d4", "b8c6", "d4b3", "c5f2", "e1f2", "c8e6",
-            "h1e1", "e8g8", "b1d2", "c6e5", "d2f3", "e5f3", "g2f3", "c7c6", "c2c3", "f7f5", "g4g5",
-            "d8d6", "e1g1", "d6h2", "g1g2", "h2h3", "b3d4", "a8e8", "c1d2", "f5f4", "d1h1", "h3h1",
-            "a1h1", "h4h3", "g2g1", "e6c8", "f3g4", "d5e3", "g4c8", "e8c8", "h1h3", "e3d5", "g5g6",
-            "d5e7", "h3h4", "f8f6", "h4g4", "c8f8", "g4g5", "b7b6", "d4f3", "f8d8", "d2c1", "d8d6",
-            "g1e1", "e7g6", "g5h5", "g6f8", "h5e5", "d6e6", "e5e6", "f6e6", "e1e6", "f8e6", "f3d4",
-            "e6d4", "c3d4", "g7g5", "f2f3", "g8f7", "c1f4", "f7g6", "f4d6", "a7a6", "d4d5", "b6b5",
-            "d5c6", "g6f5", "c6c7", "g5g4", "f3g3", "f5e6", "c7c8q", "e6d6", "c8a6", "d6c5",
-            "a6a5", "c5c4", "b2b3", "c4c5", "a5a8", "c5b4", "a8g2", "b4a3", "g3g4", "b5b4", "g2e4",
-            "a3b2", "a2a4",
-        ];
-
-        let mut pos: Zobrist<Chess, u128> = Zobrist::default();
-
-        for uci in moves {
-            let m = uci
-                .parse::<Uci>()
-                .expect("valid uci")
-                .to_move(&pos)
-                .expect("legal uci");
-
-            pos.play_unchecked(&m);
-
-            assert_eq!(pos.zobrist_hash(), pos.clone().into_inner().zobrist_hash());
+            assert_eq!(
+                pos.zobrist_hash::<Zobrist64>(EnPassantMode::Legal),
+                expected,
+                "{}",
+                fen
+            );
         }
     }
 
@@ -490,10 +433,11 @@ mod tests {
     #[test]
     fn test_variants_not_distinguished() {
         // Useful when indexing a table of opening names by Zorbist hash.
-        let chess: u128 = Chess::default().zobrist_hash();
-        let crazyhouse = crate::variant::Crazyhouse::default().zobrist_hash();
-        let three_check = crate::variant::ThreeCheck::default().zobrist_hash();
-        let king_of_the_hill = crate::variant::KingOfTheHill::default().zobrist_hash();
+        let chess: Zobrist128 = Chess::default().zobrist_hash(EnPassantMode::Legal);
+        let crazyhouse = crate::variant::Crazyhouse::default().zobrist_hash(EnPassantMode::Legal);
+        let three_check = crate::variant::ThreeCheck::default().zobrist_hash(EnPassantMode::Legal);
+        let king_of_the_hill =
+            crate::variant::KingOfTheHill::default().zobrist_hash(EnPassantMode::Legal);
         assert_eq!(chess, crazyhouse);
         assert_eq!(chess, three_check);
         assert_eq!(chess, king_of_the_hill);
@@ -504,15 +448,39 @@ mod tests {
         // 8/8/8/7k/8/8/3K4/8[ppppppppppppppppnnnnbbbbrrrrqq] w - - 0 54
         for color in Color::ALL {
             assert_ne!(
-                u64::zobrist_for_pocket(color, Role::Pawn, 16),
-                u64::zobrist_for_pocket(color, Role::Pawn, 15)
+                Zobrist64::zobrist_for_pocket(color, Role::Pawn, 16),
+                Zobrist64::zobrist_for_pocket(color, Role::Pawn, 15)
             );
             assert_ne!(
-                u64::zobrist_for_pocket(color, Role::Queen, 2),
-                u64::zobrist_for_pocket(color, Role::Queen, 1)
+                Zobrist64::zobrist_for_pocket(color, Role::Queen, 2),
+                Zobrist64::zobrist_for_pocket(color, Role::Queen, 1)
             );
         }
     }
+
+    #[cfg(feature = "nohash-hasher")]
+    #[test]
+    fn test_nohash_hasher() {
+        let mut hasher = nohash_hasher::NoHashHasher::<Zobrist128>::default();
+        Zobrist128(128).hash(&mut hasher);
+        assert_eq!(hasher.finish(), 128);
+
+        let mut hasher = nohash_hasher::NoHashHasher::<Zobrist64>::default();
+        Zobrist64(64).hash(&mut hasher);
+        assert_eq!(hasher.finish(), 64);
+
+        let mut hasher = nohash_hasher::NoHashHasher::<Zobrist32>::default();
+        Zobrist32(32).hash(&mut hasher);
+        assert_eq!(hasher.finish(), 32);
+
+        let mut hasher = nohash_hasher::NoHashHasher::<Zobrist16>::default();
+        Zobrist16(16).hash(&mut hasher);
+        assert_eq!(hasher.finish(), 16);
+
+        let mut hasher = nohash_hasher::NoHashHasher::<Zobrist8>::default();
+        Zobrist8(8).hash(&mut hasher);
+        assert_eq!(hasher.finish(), 8);
+    }
 }
 
 const PIECE_MASKS: [u128; 64 * 6 * 2] = [
