@@ -1,19 +1,3 @@
-// This file is part of the shakmaty library.
-// Copyright (C) 2017-2022 Niklas Fiekas <niklas.fiekas@backscattering.de>
-//
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 use std::{
     fs::File,
     io::{prelude::*, BufReader},
@@ -85,8 +69,9 @@ macro_rules! gen_tests {
 }
 
 gen_tests! {
-    test_random,      Chess,       "tests/random.perft",         10_000,
     test_tricky,      Chess,       "tests/tricky.perft",        100_000,
+    test_random,      Chess,       "tests/random.perft",         10_000,
+    test_chess960,    Chess,       "tests/chess960.perft",      100_000,
 }
 
 #[cfg(feature = "variant")]
